#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.navigation.portlet module

"""

from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.association import IAssociationContainer
from pyams_content.component.association.interfaces import ASSOCIATION_CONTAINER_KEY
from pyams_content.feature.navigation import IMenusContainer
from pyams_content.feature.navigation.interfaces import IMenuLinksContainer
from pyams_content.feature.navigation.portlet.interfaces import IDoubleNavigationPortletSettings, \
    ISimpleNavigationMenu, ISimpleNavigationPortletSettings
from pyams_portal.portlet import Portlet, PortletSettings, portlet_config
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import factory_config

from pyams_content import _


SIMPLE_NAVIGATION_PORTLET_NAME = 'pyams_content.portlet.navigation'
SIMPLE_NAVIGATION_ICON_CLASS = 'fas fa-bars'

SIMPLE_NAVIGATION_LINKS_KEY = f'{ASSOCIATION_CONTAINER_KEY}::links'


@factory_config(ISimpleNavigationPortletSettings)
class SimpleNavigationPortletSettings(PortletSettings):
    """Simple navigation portlet settings"""

    title = FieldProperty(ISimpleNavigationPortletSettings['title'])

    @property
    def links(self):
        """Menu links getter"""
        return IAssociationContainer(self)


@adapter_config(name='links',
                required=ISimpleNavigationPortletSettings,
                provides=IMenuLinksContainer)
def simple_navigation_links_adapter(context):
    """Simple navigation links factory"""
    return context.links


@portlet_config(permission=None)
class SimpleNavigationPortlet(Portlet):
    """Simple navigation portlet"""

    name = SIMPLE_NAVIGATION_PORTLET_NAME
    label = _("Simple navigation menu")

    settings_factory = ISimpleNavigationPortletSettings
    toolbar_css_class = SIMPLE_NAVIGATION_ICON_CLASS


DOUBLE_NAVIGATION_PORTLET_NAME = f'{SIMPLE_NAVIGATION_PORTLET_NAME}::menus'
DOUBLE_NAVIGATION_ICON_CLASS = 'fas fa-table-list'

DOUBLE_NAVIGATION_MENUS8KEY = f'{ASSOCIATION_CONTAINER_KEY}::menus'


@factory_config(IDoubleNavigationPortletSettings)
class DoubleNavigationPortletSettings(PortletSettings):
    """Double navigation portlet settings"""

    title = FieldProperty(IDoubleNavigationPortletSettings['title'])

    @property
    def menus(self):
        """Menus getter"""
        return IMenusContainer(self)


@adapter_config(name='menus',
                required=IDoubleNavigationPortletSettings,
                provides=IMenusContainer)
def double_navigation_menus_adapter(context):
    """Double navigation menus factory"""
    return context.menus


@portlet_config(permission=None)
class DoubleNavigationPortlet(Portlet):
    """Double navigation portlet"""

    name = DOUBLE_NAVIGATION_PORTLET_NAME
    label = _("Double navigation menus")

    settings_factory = IDoubleNavigationPortletSettings
    toolbar_css_class = DOUBLE_NAVIGATION_ICON_CLASS



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.extfile.manager module

This module provides components to handle external files managers.
"""

from persistent import Persistent
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.extfile.interfaces import EXTFILE_MANAGER_INFO_KEY, \
    IExtFileManagerInfo, IExtFileManagerTarget
from pyams_site.interfaces import ISiteRoot
from pyams_utils.adapter import adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config


__docformat__ = 'restructuredtext'


@factory_config(IExtFileManagerInfo)
class ExtFileManagerInfo(Persistent):
    """External files manager settings"""

    default_title_prefix = FieldProperty(IExtFileManagerInfo['default_title_prefix'])


@adapter_config(required=IExtFileManagerTarget,
                provides=IExtFileManagerInfo)
def extfile_manager_info_factory(context):
    """External files manager factory"""
    return get_annotation_adapter(context, EXTFILE_MANAGER_INFO_KEY, IExtFileManagerInfo)



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.zmi.restrictions module

This module defines components which are used to handle security restrictions.
"""

from pyramid.decorator import reify
from zope.interface import implementer

from pyams_content.interfaces import MANAGE_TOOL_PERMISSION
from pyams_content.shared.common import IBaseSharedTool
from pyams_content.shared.common.interfaces import IContributorRestrictions, \
    IContributorWorkflowRestrictions, IManagerRestrictions, IManagerWorkflowRestrictions, \
    IPrincipalRestrictions, ISharedToolRoles
from pyams_content.shared.common.zmi.interfaces import IContributorRestrictionsEditForm, \
    IManagerRestrictionsEditForm, IManagerRestrictionsGroup
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.group import Group
from pyams_form.interfaces import HIDDEN_MODE
from pyams_form.interfaces.form import IAJAXFormRenderer, IFormContent, IGroup
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces import ISecurityManager
from pyams_security_views.zmi.interfaces import IObjectSecurityMenu
from pyams_skin.interfaces.viewlet import IHelpViewletManager
from pyams_skin.viewlet.help import AlertMessage
from pyams_table.column import GetAttrColumn
from pyams_table.interfaces import IColumn, IValues
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.interfaces import MISSING_INFO
from pyams_utils.registry import get_utility
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminModalEditForm, FormGroupChecker
from pyams_zmi.helper.event import get_json_table_row_refresh_callback
from pyams_zmi.interfaces import IAdminLayer, TITLE_SPAN_BREAK
from pyams_zmi.interfaces.form import IFormTitle
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.table import I18nColumnMixin, IconColumn, Table, TableAdminView, TableElementEditor
from pyams_zmi.utils import get_object_label
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_content import _  # pylint: disable=ungrouped-imports


#
# Managers restrictions table view
#

@viewlet_config(name='managers-restrictions.menu',
                context=IBaseSharedTool, layer=IAdminLayer,
                manager=IObjectSecurityMenu, weight=200,
                permission=MANAGE_TOOL_PERMISSION)
class ManagersRestrictionsMenu(NavigationMenuItem):
    """Manager restrictions menu"""

    label = _("Managers restrictions")
    href = '#managers-restrictions.html'


class ManagersRestrictionsTable(Table):
    """Managers restrictions table"""

    def get_row_id(self, row):
        return f'{self.id}::{row[0].id}'


@adapter_config(required=(IBaseSharedTool, IAdminLayer, ManagersRestrictionsTable),
                provides=IValues)
class ManagersRestrictionsTableValues(ContextRequestViewAdapter):
    """Managers restrictions table values"""

    @property
    def values(self):
        """Managers restrictions table values getter"""
        sm = get_utility(ISecurityManager)  # pylint: disable=invalid-name
        roles = ISharedToolRoles(self.context)
        restrictions = IManagerRestrictions(self.context)
        for principal_id in roles.managers:
            yield sm.get_principal(principal_id), restrictions.get_restrictions(principal_id)


@adapter_config(name='title',
                required=(IBaseSharedTool, IAdminLayer, ManagersRestrictionsTable),
                provides=IColumn)
class ManagerTitleColumn(I18nColumnMixin, GetAttrColumn):
    """Manager title column"""

    i18n_header = _("Manager name")
    attr_name = 'title'

    weight = 10

    def get_value(self, obj):
        return super().get_value(obj[0])


@adapter_config(name='warning',
                required=(IBaseSharedTool, IAdminLayer, ManagersRestrictionsTable),
                provides=IColumn)
class ManagerWarningColumn(I18nColumnMixin, IconColumn):
    """Manager workflow warning column"""

    i18n_header = _("Show warnings")
    css_classes = {}
    weight = 20

    def get_icon_class(self, item):
        restrictions = IManagerWorkflowRestrictions(item[1])
        if restrictions.show_workflow_warning:
            return 'fas fa-check'
        return ''


@adapter_config(name='restricted',
                required=(IBaseSharedTool, IAdminLayer, ManagersRestrictionsTable),
                provides=IColumn)
class ManagerRestrictedColumn(I18nColumnMixin, IconColumn):
    """Manager workflow warning column"""

    i18n_header = _("Restricted")
    css_classes = {}
    weight = 30

    def get_icon_class(self, item):
        restrictions = IManagerWorkflowRestrictions(item[1])
        if restrictions.restricted_contents:
            return 'fas fa-check'
        return ''


@adapter_config(name='owners',
                required=(IBaseSharedTool, IAdminLayer, ManagersRestrictionsTable),
                provides=IColumn)
class ManagerOwnersColumn(I18nColumnMixin, GetAttrColumn):
    """Manager owners column"""

    i18n_header = _("Manager for")
    weight = 40

    def get_value(self, obj):
        restrictions = IManagerWorkflowRestrictions(obj[1])
        if not restrictions.owners:
            return MISSING_INFO
        sm = get_utility(ISecurityManager)
        return ', '.join((
            sm.get_principal(principal_id).title
            for principal_id in restrictions.owners
        ))


@pagelet_config(name='managers-restrictions.html',
                context=IBaseSharedTool, layer=IPyAMSLayer,
                permission=MANAGE_TOOL_PERMISSION)
class ManagersRestrictionsView(TableAdminView):
    """Managers restrictions view"""

    title = _("Managers restrictions")

    table_class = ManagersRestrictionsTable
    table_label = _("Managers list")


#
# Manager restrictions view
#

@adapter_config(required=(tuple, IAdminLayer, ManagersRestrictionsTable),
                provides=ITableElementEditor)
class ManagerRestrictionsEditor(TableElementEditor):
    """Manager restrictions editor"""

    view_name = 'manager-restrictions.html'

    @property
    def href(self):
        return absolute_url(self.view.context, self.request, self.view_name,
                            query={'principal_id': self.context[0].id})


@ajax_form_config(name='manager-restrictions.html',
                  context=IBaseSharedTool, layer=IPyAMSLayer,
                  permission=MANAGE_TOOL_PERMISSION)
@implementer(IManagerRestrictionsEditForm)
class ManagerRestrictionsEditForm(AdminModalEditForm):
    """Manager restrictions edit form"""

    @property
    def subtitle(self):
        return self.principal.title

    fields = Fields(IPrincipalRestrictions)

    @reify
    def principal_id(self):
        """Principal ID getter"""
        params = self.request.params
        return params.get('principal_id') or params.get('form.widgets.principal_id')

    @reify
    def principal(self):
        """Principal ID getter"""
        sm = get_utility(ISecurityManager)  # pylint: disable=invalid-name
        return sm.get_principal(self.principal_id)

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        principal_id = self.widgets.get('principal_id')
        if principal_id is not None:
            principal_id.mode = HIDDEN_MODE
            principal_id.value = self.principal_id


@adapter_config(required=(IBaseSharedTool, IAdminLayer, IManagerRestrictionsEditForm),
                provides=IFormTitle)
def base_shared_tool_manager_restrictions_form_title(context, request, form):
    """Base shared tool manager restrictions edit form title getter"""
    translate = request.localizer.translate
    return TITLE_SPAN_BREAK.format(
        get_object_label(context, request, form),
        translate(_("Manager restrictions")))


@adapter_config(required=(IBaseSharedTool, IAdminLayer, IManagerRestrictionsEditForm),
                provides=IFormContent)
def manager_restrictions_edit_form_content(context, request, form):
    """Manager restrictions edit form content getter"""
    return IManagerRestrictions(context).get_restrictions(form.principal.id)


@adapter_config(required=(IBaseSharedTool, IAdminLayer, IManagerRestrictionsEditForm),
                provides=IAJAXFormRenderer)
class ManagerRestrictionsEditFormRenderer(ContextRequestViewAdapter):
    """Manager restrictions edit form renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if not changes:
            return None
        sm = get_utility(ISecurityManager)
        restrictions = IManagerRestrictions(self.context)
        principal_id = self.view.principal_id
        context = sm.get_principal(principal_id), restrictions.get_restrictions(principal_id)
        return {
            'callbacks': [
                get_json_table_row_refresh_callback(self.context, self.request,
                                                    ManagersRestrictionsTable,
                                                    context)
            ]
        }


@adapter_config(name='workflow',
                required=(IBaseSharedTool, IAdminLayer, IManagerRestrictionsEditForm),
                provides=IGroup)
class ManagerRestrictionsWorkflowGroup(Group):
    """Manager restrictions workflow group"""

    legend = _("Principal restrictions")
    fields = Fields(IManagerWorkflowRestrictions).select('show_workflow_warning')


@adapter_config(required=(IBaseSharedTool, IAdminLayer, ManagerRestrictionsWorkflowGroup),
                provides=IFormContent)
def get_manager_restrictions_workflow_group_content(context, request, group):
    """Manager restrictions workflow group content getter"""
    return IManagerWorkflowRestrictions(group.parent_form.get_content())


@adapter_config(name='restrictions',
                required=(IBaseSharedTool, IAdminLayer, ManagerRestrictionsWorkflowGroup),
                provides=IGroup)
@implementer(IManagerRestrictionsGroup)
class ManagerRestrictionInnerGroup(FormGroupChecker):
    """Manager restrictions inner group"""

    fields = Fields(IManagerWorkflowRestrictions).omit('show_workflow_warning')
    weight = 10

    checker_fieldname = 'restricted_contents'


@viewlet_config(name='restrictions.help',
                context=IBaseSharedTool, layer=IAdminLayer, view=ManagerRestrictionInnerGroup,
                manager=IHelpViewletManager, weight=10)
class ManagerRestrictionsHelp(AlertMessage):
    """Manager restrictions help"""

    status = 'info'
    _message = _("If restrictions are enabled for this manager, he will only be able to "
                 "manage contents matching at least one of these properties!")

    message_renderer = 'markdown'


#
# Contributors restrictions table view
#

@viewlet_config(name='contributors-restrictions.menu',
                context=IBaseSharedTool, layer=IAdminLayer,
                manager=IObjectSecurityMenu, weight=210,
                permission=MANAGE_TOOL_PERMISSION)
class ContributorsRestrictionsMenu(NavigationMenuItem):
    """Contributor restrictions menu"""

    label = _("Contributors restrictions")
    href = '#contributors-restrictions.html'


class ContributorsRestrictionsTable(Table):
    """Contributors restrictions table"""

    def get_row_id(self, row):
        return f'{self.id}::{row[0].id}'


@adapter_config(required=(IBaseSharedTool, IAdminLayer, ContributorsRestrictionsTable),
                provides=IValues)
class ContributorsRestrictionsTableValues(ContextRequestViewAdapter):
    """Contributors restrictions table values"""

    @property
    def values(self):
        """Contributors restrictions table values getter"""
        sm = get_utility(ISecurityManager)  # pylint: disable=invalid-name
        roles = ISharedToolRoles(self.context)
        restrictions = IContributorRestrictions(self.context)
        for principal_id in roles.contributors:
            yield sm.get_principal(principal_id), restrictions.get_restrictions(principal_id)


@adapter_config(name='title',
                required=(IBaseSharedTool, IAdminLayer, ContributorsRestrictionsTable),
                provides=IColumn)
class ContributorTitleColumn(I18nColumnMixin, GetAttrColumn):
    """Contributor title column"""

    i18n_header = _("Contributor name")
    attr_name = 'title'

    weight = 10

    def get_value(self, obj):
        return super().get_value(obj[0])


@adapter_config(name='warning',
                required=(IBaseSharedTool, IAdminLayer, ContributorsRestrictionsTable),
                provides=IColumn)
class ContributorWarningColumn(I18nColumnMixin, IconColumn):
    """Contributor workflow warning column"""

    i18n_header = _("Show warnings")
    css_classes = {}
    weight = 20

    def get_icon_class(self, item):
        restrictions = IContributorWorkflowRestrictions(item[1])
        if restrictions.show_workflow_warning:
            return 'fas fa-check'
        return ''


@adapter_config(name='substitutes',
                required=(IBaseSharedTool, IAdminLayer, ContributorsRestrictionsTable),
                provides=IColumn)
class ContributorSubstitutesColumn(I18nColumnMixin, GetAttrColumn):
    """Contributor substitutes column"""

    i18n_header = _("Substitute of")
    weight = 30

    def get_value(self, obj):
        restrictions = IContributorWorkflowRestrictions(obj[1])
        if not restrictions.owners:
            return MISSING_INFO
        sm = get_utility(ISecurityManager)
        return ', '.join((
            sm.get_principal(principal_id).title
            for principal_id in restrictions.owners
        ))


@pagelet_config(name='contributors-restrictions.html',
                context=IBaseSharedTool, layer=IPyAMSLayer,
                permission=MANAGE_TOOL_PERMISSION)
class ContributorsRestrictionsView(TableAdminView):
    """Contributors restrictions view"""

    title = _("Contributors restrictions")

    table_class = ContributorsRestrictionsTable
    table_label = _("Contributors list")


#
# Contributor restrictions view
#

@adapter_config(required=(tuple, IAdminLayer, ContributorsRestrictionsTable),
                provides=ITableElementEditor)
class ContributorRestrictionsEditor(TableElementEditor):
    """Contributor restrictions editor"""

    view_name = 'contributor-restrictions.html'

    @property
    def href(self):
        return absolute_url(self.view.context, self.request, self.view_name,
                            query={'principal_id': self.context[0].id})


@ajax_form_config(name='contributor-restrictions.html',
                  context=IBaseSharedTool, layer=IPyAMSLayer,
                  permission=MANAGE_TOOL_PERMISSION)
@implementer(IContributorRestrictionsEditForm)
class ContributorRestrictionsEditForm(AdminModalEditForm):
    """Contributor restrictions edit form"""

    @property
    def subtitle(self):
        return self.principal.title

    fields = Fields(IPrincipalRestrictions)

    @reify
    def principal_id(self):
        """Principal ID getter"""
        params = self.request.params
        return params.get('principal_id') or params.get('form.widgets.principal_id')

    @reify
    def principal(self):
        """Principal ID getter"""
        sm = get_utility(ISecurityManager)  # pylint: disable=invalid-name
        return sm.get_principal(self.principal_id)

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        principal_id = self.widgets.get('principal_id')
        if principal_id is not None:
            principal_id.mode = HIDDEN_MODE
            principal_id.value = self.principal_id


@adapter_config(required=(IBaseSharedTool, IAdminLayer, IContributorRestrictionsEditForm),
                provides=IFormTitle)
def base_shared_tool_contributor_restrictions_form_title(context, request, form):
    """Base shared tool contributor restrictions edit form title getter"""
    translate = request.localizer.translate
    return TITLE_SPAN_BREAK.format(
        get_object_label(context, request, form),
        translate(_("Contributor restrictions")))


@adapter_config(required=(IBaseSharedTool, IAdminLayer, IContributorRestrictionsEditForm),
                provides=IFormContent)
def contributor_restrictions_edit_form_content(context, request, form):
    """Contributor restrictions edit form content getter"""
    return IContributorRestrictions(context).get_restrictions(form.principal.id)


@adapter_config(required=(IBaseSharedTool, IAdminLayer, IContributorRestrictionsEditForm),
                provides=IAJAXFormRenderer)
class ContributorRestrictionsEditFormRenderer(ContextRequestViewAdapter):
    """Contributor restrictions edit form renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if not changes:
            return None
        sm = get_utility(ISecurityManager)
        restrictions = IContributorRestrictions(self.context)
        principal_id = self.view.principal_id
        context = sm.get_principal(principal_id), restrictions.get_restrictions(principal_id)
        return {
            'callbacks': [
                get_json_table_row_refresh_callback(self.context, self.request,
                                                    ContributorsRestrictionsTable,
                                                    context)
            ]
        }


@adapter_config(name='workflow',
                required=(IBaseSharedTool, IAdminLayer, IContributorRestrictionsEditForm),
                provides=IGroup)
class ContributorRestrictionsWorkflowGroup(Group):
    """Contributor_restrictions workflow group"""

    legend = _("Principal restrictions")
    fields = Fields(IContributorWorkflowRestrictions)


@adapter_config(required=(IBaseSharedTool, IAdminLayer, ContributorRestrictionsWorkflowGroup),
                provides=IFormContent)
def base_shared_tool_contributor_restrictions_form_content(context, request, form):
    """Base shared tool contributor restrictions edit form content getter"""
    return IContributorWorkflowRestrictions(form.parent_form.get_content())



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.topic.zmi module

This module defines topics management components.
"""

from zope.interface import Interface

from pyams_content.shared.common.zmi.types.content import TypedSharedContentPropertiesEditForm
from pyams_content.shared.topic import IWfTopic
from pyams_form.ajax import ajax_form_config
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_utils.adapter import adapter_config
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IContentManagementMenu, IMenuHeader


__docformat__ = 'restructuredtext'

from pyams_content import _


@adapter_config(required=(IWfTopic, IAdminLayer, Interface, IContentManagementMenu),
                provides=IMenuHeader)
def topic_management_menu_header(context, request, view, manager):
    """Topic management menu header"""
    return request.localizer.translate(_("Topic management"))


@ajax_form_config(name='properties.html',
                  context=IWfTopic, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class TopicPropertiesEditForm(TypedSharedContentPropertiesEditForm):
    """Topic properties edit form"""

    interface = IWfTopic



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.view.skin module

This module defines a simple view which is used to get a preview of view items.
"""

from pyams_content.shared.view import IWfView
from pyams_layer.interfaces import IPyAMSLayer
from pyams_pagelet.pagelet import pagelet_config
from pyams_portal.skin.page import PortalContextPreviewPage
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_template.template import template_config

__docformat__ = 'restructuredtext'


@pagelet_config(name='preview.html',
                context=IWfView, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
@template_config(template='templates/preview.pt', layer=IPyAMSLayer)
class ViewPreviewPage(PortalContextPreviewPage):
    """Shared view preview page"""

    @property
    def items(self):
        return self.context.get_results(self.context, ignore_cache=True, request=self.request)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.alert.interfaces module

This module defines interfaces of alerts tool and contents.
"""

from collections import OrderedDict
from enum import Enum

from zope.container.constraints import contains
from zope.container.interfaces import IContainer
from zope.interface import Invalid, invariant
from zope.location.interfaces import ILocation
from zope.schema import Bool, Choice, Int, TextLine, URI

from pyams_content.reference.pictogram import PICTOGRAM_VOCABULARY
from pyams_content.shared.common import ISharedContent, IWfSharedContent
from pyams_content.shared.common.interfaces import ISharedTool
from pyams_i18n.schema import I18nTextField, I18nTextLineField
from pyams_sequence.interfaces import IInternalReferencesList
from pyams_sequence.schema import InternalReferenceField, InternalReferencesListField
from pyams_utils.schema import ColorField

__docformat__ = 'restructuredtext'

from pyams_content import _

ALERT_TYPES_MANAGER_ANNOTATION_KEY = 'pyams_content.alerts.types'
ALERT_TYPES_VOCABULARY = 'pyams_content.alerts.types'


class IAlertType(ILocation):
    """Alert type interface"""

    visible = Bool(title=_("Visible?"),
                   description=_("An hidden alert type can't be assigned to new alerts"),
                   required=True,
                   default=True)

    label = I18nTextLineField(title=_("Label"),
                              required=True)

    backoffice_label = I18nTextLineField(title=_("Back-office label"),
                                         description=_("Optional label used in management pages "
                                                       "instead of default label"),
                                         required=False)

    pictogram = Choice(title=_("Pictogram"),
                       description=_("Pictogram associated with this alert type"),
                       vocabulary=PICTOGRAM_VOCABULARY,
                       required=False)

    color = ColorField(title=_("Color"),
                       description=_("Base color associated with this alert type"),
                       required=False,
                       default='dc3545')


class IAlertTypesManager(IContainer):
    """Alert types manager interface"""

    contains(IAlertType)

    def get_visible_items(self):
        """Visible alert types iterator"""


ALERT_CONTENT_TYPE = 'alert'
ALERT_CONTENT_NAME = _("Alert")


class IWfAlert(IWfSharedContent, IInternalReferencesList):
    """Alert interface"""

    alert_type = Choice(title=_("Alert type"),
                        description=_("Alert type can affect renderer alert style"),
                        required=True,
                        vocabulary=ALERT_TYPES_VOCABULARY)

    def get_alert_type(self):
        """Alert type getter"""

    body = I18nTextField(title=_("Message content"),
                         description=_("Message body"),
                         required=False)

    reference = InternalReferenceField(title=_("Internal reference"),
                                       description=_("Internal link target reference. You can "
                                                     "search a reference using '+' followed by "
                                                     "internal number, or by entering text "
                                                     "matching content title"),
                                       required=False)

    external_url = URI(title=_("External URL"),
                       description=_("Alternate external URL"),
                       required=False)

    @invariant
    def check_url(self):
        if self.reference and self.external_url:
            raise Invalid(_("You can't set internal reference and external URI simultaneously!"))

    references = InternalReferencesListField(title=_("Concerned contents"),
                                             description=_("If any, these contents will "
                                                           "automatically display this alert"),
                                             required=False)

    maximum_interval = Int(title=_("Maximum interval"),
                           description=_("Maximum interval between alert displays on a given "
                                         "device, given in hours; set to 0 to always display "
                                         "the alert"),
                           required=True,
                           min=0,
                           default=48)


class IAlert(ISharedContent):
    """Workflow managed alert interface"""


class IAlertManager(ISharedTool):
    """Alert manager interface"""

    def find_context_alerts(self, context=None, request=None):
        """Find alerts associated with given context"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.view.interfaces module

This module defines views interfaces.
"""

from collections import OrderedDict

from zope.interface import Attribute
from zope.schema import Bool, Choice, Int, Set
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_content.shared.common.interfaces import ISharedContent, ISharedTool, IWfSharedContent, \
    VIEWS_SHARED_CONTENT_TYPES_VOCABULARY
from pyams_content.shared.common.interfaces.types import ALL_DATA_TYPES_VOCABULARY

__docformat__ = 'restructuredtext'

from pyams_content import _


VIEW_CONTENT_TYPE = 'view'
VIEW_CONTENT_NAME = _('View')


RELEVANCE_ORDER = 'relevance'
CREATION_DATE_ORDER = 'created_date'
UPDATE_DATE_ORDER = 'modified_date'
PUBLICATION_DATE_ORDER = 'publication_date'
FIRST_PUBLICATION_DATE_ORDER = 'first_publication_date'
CONTENT_PUBLICATION_DATE_ORDER = 'content_publication_date'
VISIBLE_PUBLICATION_DATE_ORDER = 'visible_publication_date'
EXPIRATION_DATE_ORDER = 'expiration_date'


VIEW_ORDERS = OrderedDict((
    (CREATION_DATE_ORDER, _("Published version creation date")),
    (UPDATE_DATE_ORDER, _("Published version last update date")),
    (PUBLICATION_DATE_ORDER, _("Current version publication date")),
    (FIRST_PUBLICATION_DATE_ORDER, _("Current version first publication date")),
    (CONTENT_PUBLICATION_DATE_ORDER, _("Content first publication date")),
    (VISIBLE_PUBLICATION_DATE_ORDER, _("Visible publication date")),
    (EXPIRATION_DATE_ORDER, _("Expiration date"))
))

VIEW_ORDER_VOCABULARY = SimpleVocabulary([
    SimpleTerm(v, title=t)
    for v, t in VIEW_ORDERS.items()
])


USER_VIEW_ORDERS = OrderedDict((
    (RELEVANCE_ORDER, _("Relevance (on user search; if not on visible publication date)")),
    (CREATION_DATE_ORDER, _("Published version creation date")),
    (UPDATE_DATE_ORDER, _("Published version last update date")),
    (PUBLICATION_DATE_ORDER, _("Current version publication date")),
    (FIRST_PUBLICATION_DATE_ORDER, _("Current version first publication date")),
    (CONTENT_PUBLICATION_DATE_ORDER, _("Content first publication date")),
    (VISIBLE_PUBLICATION_DATE_ORDER, _("Visible publication date")),
    (EXPIRATION_DATE_ORDER, _("Expiration date"))
))

USER_VIEW_ORDER_VOCABULARY = SimpleVocabulary([
    SimpleTerm(v, title=t)
    for v, t in USER_VIEW_ORDERS.items()
])


class IWfView(IWfSharedContent):
    """View interface"""

    select_context_path = Bool(title=_("Select context path?"),
                               description=_("If 'yes', only contents located inside context "
                                             "will be selected"),
                               required=True,
                               default=False)

    def get_content_path(self, context):
        """Get context path internal ID"""

    select_context_type = Bool(title=_("Select context type?"),
                               description=_("If 'yes', content type will be extracted from "
                                             "context"),
                               required=True,
                               default=False)

    selected_content_types = Set(title=_("Other content types"),
                                 description=_("Selected content types; leave empty for all"),
                                 value_type=Choice(vocabulary=VIEWS_SHARED_CONTENT_TYPES_VOCABULARY),
                                 required=False)

    def get_ignored_types(self):
        """Get content typed which are excluded from views (except if specified explicitly at runtime)"""

    def get_content_types(self, context):
        """Get content types for given context"""

    select_context_datatype = Bool(title=_("Select context data type?"),
                                   description=_("If 'yes', content data type (if available) "
                                                 "will be extracted from context"),
                                   required=True,
                                   default=False)

    selected_datatypes = Set(title=_("Other data types"),
                             description=_("Selected data types; leave empty for all"),
                             value_type=Choice(vocabulary=ALL_DATA_TYPES_VOCABULARY),
                             required=False)

    def get_data_types(self, context):
        """Get data types for given context"""

    excluded_content_types = Set(title=_("Excluded content types"),
                                 description=_("Excluded content types; leave empty for all"),
                                 value_type=Choice(vocabulary=VIEWS_SHARED_CONTENT_TYPES_VOCABULARY),
                                 required=False)

    def get_excluded_content_types(self, context):
        """Get excluded content types for given context"""

    excluded_datatypes = Set(title=_("Excluded data types"),
                             description=_("Excluded data types; leave empty for all"),
                             value_type=Choice(vocabulary=ALL_DATA_TYPES_VOCABULARY),
                             required=False)

    allow_user_params = Bool(title=_("Allow user params?"),
                             description=_("If 'no', additional user params provided through request "
                                           "URL will not be analyzed"),
                             required=True,
                             default=True)

    def get_excluded_data_types(self, context):
        """Get excluded data types for given context"""

    order_by = Choice(title=_("Order by"),
                      description=_("Property to use to sort results; publication date can be "
                                    "different from first publication date for contents which "
                                    "have been retired and re-published with a different "
                                    "publication date"),
                      vocabulary=VIEW_ORDER_VOCABULARY,
                      required=True,
                      default=VISIBLE_PUBLICATION_DATE_ORDER)

    reversed_order = Bool(title=_("Reversed order?"),
                          description=_("If 'yes', items order will be reversed"),
                          required=True,
                          default=True)

    limit = Int(title=_("Results count limit"),
                description=_("Maximum number of results that the view may retrieve"),
                max=1000,
                required=False)

    age_limit = Int(title=_("Results age limit"),
                    description=_("If specified, contents whose publication date (given in "
                                  "days) is older than this limit will be ignored"),
                    required=False)

    is_using_context = Attribute("Check if view is using context settings")

    def get_results(self, context, sort_index=None, reverse=True, limit=None,
                    start=0, length=999, ignore_cache=False, get_count=False,
                    request=None, **kwargs):
        """Get query results"""


class IView(ISharedContent):
    """Workflow managed view interface"""


class IViewManager(ISharedTool):
    """View manager interface"""



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.zmi.content module

This module provides common management components for shared contents.
"""

from zope.interface import Interface

from pyams_content.shared.common import IBaseSharedTool
from pyams_content.shared.common.interfaces import ISharedContent, IWfSharedContent, IWfSharedContentRoles
from pyams_content.shared.common.interfaces.types import IWfTypedSharedContent
from pyams_content.zmi.interfaces import IDashboardColumn, IDashboardContentNumber, \
    IDashboardContentOwner, IDashboardContentType, ISiteRootDashboardContentType
from pyams_i18n.interfaces import II18n
from pyams_security.utility import get_principal
from pyams_security_views.interfaces.login import ILoginView
from pyams_sequence.interfaces import ISequentialIdTarget
from pyams_skin.interfaces.view import IModalPage
from pyams_skin.interfaces.viewlet import IHeaderViewletManager
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.date import format_datetime
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_viewlet.viewlet import EmptyViewlet, Viewlet, viewlet_config
from pyams_workflow.interfaces import IWorkflow, IWorkflowPublicationInfo, IWorkflowState, \
    IWorkflowStateLabel, IWorkflowVersions
from pyams_zmi.interfaces import IAdminLayer, IObjectHint, IObjectLabel
from pyams_zmi.zmi.viewlet.header import ContentHeaderViewlet

__docformat__ = 'restructuredtext'

from pyams_content import _


def shared_content_version_getter(context, request, view, interface):
    """Shared content version generic interface getter"""
    versions = IWorkflowVersions(context, None)
    if versions is not None:
        version = versions.get_last_versions()[0]
        return request.registry.queryMultiAdapter((version, request, view), interface)


@adapter_config(required=(IWfSharedContent, IAdminLayer, Interface),
                provides=IObjectHint)
def wf_shared_content_hint(context, request, view):
    """Workflow-managed shared content hint"""
    translate = request.localizer.translate
    return translate(context.content_name)


@adapter_config(required=(ISharedContent, IAdminLayer, Interface),
                provides=IObjectHint)
def shared_content_hint(context, request, view):
    """Shared content hint"""
    return shared_content_version_getter(context, request, view, IObjectHint)


@adapter_config(required=(IWfSharedContent, IAdminLayer, Interface),
                provides=IObjectLabel)
def wf_shared_content_label(context, request, view):
    """Workflow-managed shared content label"""
    return II18n(context).query_attribute('title', request=request)


@adapter_config(required=(ISharedContent, IAdminLayer, Interface),
                provides=IObjectLabel)
def shared_content_label(context, request, view):
    """Shared content label"""
    return shared_content_version_getter(context, request, view, IObjectLabel)


@adapter_config(required=(IWfSharedContent, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentType)
def wf_shared_content_type(context, request, column):
    """Workflow-managed shared content type"""
    if IWfTypedSharedContent.providedBy(context):
        data_type = context.get_data_type()
        if data_type is not None:
            i18n = II18n(data_type)
            return i18n.query_attributes_in_order(('backoffice_label', 'label'),
                                                  request=request)
    return None


@adapter_config(required=(ISharedContent, IAdminLayer, Interface),
                provides=IDashboardContentType)
def shared_content_type(context, request, view):
    """Shared content label"""
    return shared_content_version_getter(context, request, view, IDashboardContentType)


@adapter_config(required=(IWfSharedContent, IAdminLayer, IDashboardColumn),
                provides=ISiteRootDashboardContentType)
def wf_site_root_shared_content_type(context, request, column):
    """Site root workflow-managed shared content type"""
    translate = request.localizer.translate
    data_type = wf_shared_content_type(context, request, column)
    content_name = translate(context.content_name)
    if data_type is None:
        return content_name
    return f'{content_name} ({data_type})'


@adapter_config(required=(ISharedContent, IAdminLayer, IDashboardColumn),
                provides=ISiteRootDashboardContentType)
def site_root_shared_content_type(context, request, view):
    """Site root shared content type"""
    return shared_content_version_getter(context, request, view, ISiteRootDashboardContentType)


@adapter_config(required=(IWfSharedContent, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentNumber)
def wf_shared_content_number(context, request, column):
    """Shared content dashboard number getter"""
    target = get_parent(context, ISequentialIdTarget)
    return request.registry.queryMultiAdapter((target, request, column),
                                              IDashboardContentNumber)


@adapter_config(required=(IWfSharedContent, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentOwner)
def wf_shared_content_owner(context, request, column):
    """Workflow-managed shared content dashboard owner getter"""
    owner = IWfSharedContentRoles(context).owner
    if owner:
        return get_principal(request, next(iter(owner))).title
    return None


@adapter_config(required=(ISharedContent, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentOwner)
def shared_content_owner(context, request, view):
    """Shared content owner getter"""
    return shared_content_version_getter(context, request, view, IDashboardContentOwner)


@viewlet_config(name='pyams.content_header',
                context=IWfSharedContent, layer=IAdminLayer,
                manager=IHeaderViewletManager, weight=10)
@template_config(template='templates/content-header.pt')
class SharedContentHeaderViewlet(ContentHeaderViewlet):
    """Shared content header viewlet"""

    @property
    def parent_target_url(self):
        """Parent target URL"""
        tool = get_parent(self.context, IBaseSharedTool)
        if tool is None:
            return None
        return absolute_url(tool, self.request, 'admin#dashboard.html')

    @property
    def owner(self):
        """Owner getter"""
        owner = IWfSharedContentRoles(self.context).owner
        if owner:
            translate = self.request.localizer.translate
            return translate(_("from {}")).format(
                get_principal(self.request, next(iter(owner))).title)


@viewlet_config(name='pyams.content_header',
                context=IWfSharedContent, layer=IAdminLayer, view=ILoginView,
                manager=IHeaderViewletManager, weight=10)
class SharedContentHeaderLoginViewViewlet(EmptyViewlet):
    """Shared content header viewlet on login view"""


@viewlet_config(name='workflow-status',
                context=IWfSharedContent, layer=IAdminLayer,
                manager=IHeaderViewletManager, weight=20)
@template_config(template='templates/content-workflow.pt')
class SharedContentWorkflowStatus(Viewlet):
    """Shared content workflow status"""

    @property
    def version(self):
        """Version getter"""
        return IWorkflowState(self.context).version_id

    @staticmethod
    def create_link(href, title, css_class):
        """Link creation"""
        return f'<a href="{href}" class="{css_class}">{title}</a>'

    @property
    def state(self):
        context = self.context
        request = self.request
        registry = request.registry
        translate = request.localizer.translate
        result = []
        workflow = IWorkflow(context)
        state = IWorkflowState(context)
        versions = IWorkflowVersions(context)
        # init state format
        state_format = translate(_('{state} <span class="px-1">by</span> {principal}'))
        if state.state in workflow.waiting_states:
            if state.state_urgency:
                state_format = state_format.replace(
                    '{state}',
                    '{state} <i class="fas fa-fw fa-exclamation-triangle text-danger"></i>')
        elif state.state in workflow.published_states:
            pub_info = IWorkflowPublicationInfo(context, None)
            if (pub_info is not None) and not pub_info.is_published():
                state_format = state_format.replace(
                    '{state}',
                    '{{state}} <i class="fas fa-fw fa-hourglass-half font-xs '
                    'text-danger opacity-75 hint align-base" '
                    'data-ams-hint-offset="5" title="{0}"></i>'.format(
                        translate(_("Content publication start date is not passed yet"))))
        state_class = 'text-danger'
        state_format = state_format.replace(
            '{state}',
            f'<span class="{state_class}">{{state}}</span>')
        # init principal format
        principal_class = 'text-danger' \
            if state.state in (workflow.update_states | workflow.waiting_states) \
            else 'txt-color-text'
        state_format = state_format.replace(
            '{principal}',
            f'<span class="{principal_class}">{{principal}}</span>')
        # get state label
        state_label = registry.queryAdapter(workflow, IWorkflowStateLabel, name=state.state)
        if state_label is None:
            state_label = registry.queryAdapter(workflow, IWorkflowStateLabel)
        if state_label is None:
            wf_state = state_format.format(
                state=translate(workflow.get_state_label(state.state)),
                principal=get_principal(request, state.state_principal).title)
        else:
            wf_state = state_format.format(
                state=state_label.get_label(context, request, format=False),
                principal=get_principal(request, state.state_principal).title)
        result.append(wf_state)
        result.append(translate(_("since {date}")).format(
            date=format_datetime(state.state_date, request=request)))
        # check for links to other versions
        visible_versions = sorted(versions.get_versions(workflow.visible_states),
                                  key=lambda x: IWorkflowState(x).version_id)
        draft_versions = sorted(versions.get_versions(workflow.update_states),
                                key=lambda x: IWorkflowState(x).version_id)
        if (state.version_id > 1) or (state.state != workflow.initial_state):
            targets = set()
            if visible_versions:
                target = visible_versions[0]
                if (target is not context) and (target not in targets):
                    result.append(self.create_link(**{
                        'title': translate(_("access published version")),
                        'href': absolute_url(target, request, 'admin'),
                        'css_class': 'text-primary'
                    }))
                    targets.add(target)
            if draft_versions:
                target = draft_versions[0]
                if (target is not context) and (target not in targets):
                    result.append(self.create_link(**{
                        'title': translate(_("access new version")),
                        'href': absolute_url(target, request, 'admin'),
                        'css_class': 'text-primary'
                    }))
                    targets.add(target)
            else:
                waiting_versions = sorted(versions.get_versions(workflow.waiting_states),
                                          key=lambda x: IWorkflowState(x).version_id)
                if waiting_versions:
                    target = waiting_versions[-1]
                    if (target is not context) and (target not in targets):
                        result.append(self.create_link(**{
                            'title': translate(_("access waiting version")),
                            'href': absolute_url(target, request, 'admin'),
                            'css_class': 'text-primary'
                        }))
                        targets.add(target)
            if (state.version_id > 1) and \
                    (state.state not in (workflow.retired_states | workflow.archived_states)):
                retired_versions = sorted(filter(
                        lambda x: IWorkflowState(x).version_id < state.version_id,
                        versions.get_versions(workflow.retired_states)),
                    key=lambda x: IWorkflowState(x).version_id)
                if retired_versions:
                    target = retired_versions[-1]
                    if (target is not context) and (target not in targets):
                        result.append(self.create_link(**{
                            'title': translate(_("access retired version")),
                            'href': absolute_url(target, request, 'admin'),
                            'css_class': 'text-primary'
                        }))
                else:
                    archived_versions = sorted(filter(
                            lambda x: IWorkflowState(x).version_id < state.version_id,
                            versions.get_versions(workflow.archived_states)),
                        key=lambda x: IWorkflowState(x).version_id)
                    if archived_versions:
                        target = archived_versions[-1]
                        if (target is not context) and (target not in targets):
                            result.append(self.create_link(**{
                                'title': translate(_("access archived version")),
                                'href': absolute_url(target, request, 'admin'),
                                'css_class': 'text-primary'
                            }))
        return '<span class="px-2">|</span>'.join(result)


@viewlet_config(name='workflow-status',
                context=IWfSharedContent, layer=IAdminLayer, view=IModalPage,
                manager=IHeaderViewletManager, weight=20)
@viewlet_config(name='workflow-status',
                context=IWfSharedContent, layer=IAdminLayer, view=ILoginView,
                manager=IHeaderViewletManager, weight=20)
class SharedContentWorkflowModalStatus(EmptyViewlet):
    """Shared content workflow modal status"""



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.view.zmi.references module

This module defines component used for view references management interface.
"""

from pyams_content.shared.view import IWfView
from pyams_content.shared.view.interfaces.settings import IViewInternalReferencesSettings
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminEditForm
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.viewlet import IPropertiesMenu
from pyams_zmi.zmi.viewlet.menu import NavigationMenuDivider, NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_content import _


@viewlet_config(name='references.divider',
                context=IWfView, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=299,
                permission=VIEW_SYSTEM_PERMISSION)
class ViewReferencesMenuDivider(NavigationMenuDivider):
    """View references menu divider"""


@viewlet_config(name='references.menu',
                context=IWfView, layer=IAdminLayer,
                manager=IPropertiesMenu, weight=300,
                permission=VIEW_SYSTEM_PERMISSION)
class ViewReferencesMenu(NavigationMenuItem):
    """View references menu"""

    label = _("References")
    href = '#references.html'


@ajax_form_config(name='references.html',
                  context=IWfView, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class ViewReferencesEditForm(AdminEditForm):
    """View references settings edit form"""

    title = _("References settings")
    legend = _("View internal references settings")

    fields = Fields(IViewInternalReferencesSettings)



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.glossary.skin.term module

This module provides custom glossary terms renderers.
"""

__docformat__ = 'restructuredtext'

from zope.interface import Interface

from pyams_content.component.illustration import IIllustration
from pyams_content.feature.glossary.skin import IThesaurusTermRenderer
from pyams_i18n.interfaces import II18n
from pyams_layer.interfaces import IPyAMSUserLayer
from pyams_template.template import template_config
from pyams_thesaurus.extension.html import IThesaurusTermHTMLInfo
from pyams_thesaurus.interfaces.term import IThesaurusTerm
from pyams_utils.adapter import adapter_config
from pyams_viewlet.viewlet import ViewContentProvider


@adapter_config(name='header',
                required=(IThesaurusTerm, IPyAMSUserLayer, Interface),
                provides=IThesaurusTermRenderer)
@template_config(template='templates/term-header.pt',
                 layer=IPyAMSUserLayer)
class ThesaurusTermHeaderRenderer(ViewContentProvider):
    """Thesaurus term header renderer"""

    weight = 5


@adapter_config(name='illustration',
                required=(IThesaurusTerm, IPyAMSUserLayer, Interface),
                provides=IThesaurusTermRenderer)
@template_config(template='templates/term-illustration.pt',
                 layer=IPyAMSUserLayer)
class ThesaurusTermIllustrationRenderer(ViewContentProvider):
    """Thesaurus term illustration renderer"""

    weight = 10

    illustration = None

    def update(self):
        super().update()
        self.illustration = IIllustration(self.context, None)

    def render(self, template_name=''):
        if not (self.illustration and self.illustration.data):
            return ''
        return super().render(template_name)


@adapter_config(name='body',
                required=(IThesaurusTerm, IPyAMSUserLayer, Interface),
                provides=IThesaurusTermRenderer)
@template_config(template='templates/term-body.pt',
                 layer=IPyAMSUserLayer)
class ThesaurusTermBodyRenderer(ViewContentProvider):
    """Thesaurus term body renderer"""

    weight = 20

    body = None

    def update(self):
        super().update()
        info = IThesaurusTermHTMLInfo(self.context, None)
        if info is not None:
            self.body = II18n(info).query_attribute('description', request=self.request)


@adapter_config(name='associations',
                required=(IThesaurusTerm, IPyAMSUserLayer, Interface),
                provides=IThesaurusTermRenderer)
@template_config(template='templates/term-associations.pt',
                 layer=IPyAMSUserLayer)
class ThesaurusTermAssociationsRenderer(ViewContentProvider):
    """Thesaurus term associations renderer"""

    weight = 50


@adapter_config(name='footer',
                required=(IThesaurusTerm, IPyAMSUserLayer, Interface),
                provides=IThesaurusTermRenderer)
@template_config(template='templates/term-footer.pt',
                 layer=IPyAMSUserLayer)
class ThesaurusTermFooterRenderer(ViewContentProvider):
    """Thesaurus term footer renderer"""

    weight = 90



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.skin.url module

This module defines adapters used to manage shared contents canonical
and relative URLs.
"""

from pyramid.encode import urlencode, url_quote
from pyramid.url import QUERY_SAFE

from pyams_content.shared.common import ISharedContent, IWfSharedContent
from pyams_content.shared.common.interfaces.types import IWfTypedSharedContent
from pyams_layer.interfaces import IPyAMSUserLayer
from pyams_sequence.interfaces import ISequentialIdInfo
from pyams_utils.adapter import ContextRequestAdapter, adapter_config
from pyams_utils.interfaces.url import ICanonicalURL, IRelativeURL
from pyams_utils.url import absolute_url, canonical_url, relative_url

__docformat__ = 'restructuredtext'


def get_quoted_query(query):
    """Query checker"""
    if not query:
        return None
    if isinstance(query, str):
        return url_quote(query, QUERY_SAFE)
    return urlencode(query, doseq=True)


#
# Canonical URL adapters
#

@adapter_config(required=(IWfSharedContent, IPyAMSUserLayer),
                provides=ICanonicalURL)
class WfSharedContentCanonicalURL(ContextRequestAdapter):
    """Workflow managed shared content canonical URL adapter"""

    def get_url(self, view_name=None, query=None):
        """Canonical URL getter"""
        query = get_quoted_query(query)
        return absolute_url(self.request.root, self.request,
                            f"+/{ISequentialIdInfo(self.context).get_base_oid().strip()}"
                            f"::{self.context.content_url}"
                            f"{'/{}'.format(view_name) if view_name else '.html'}"
                            f"{'?{}'.format(query) if query else ''}")


@adapter_config(required=(IWfTypedSharedContent, IPyAMSUserLayer),
                provides=ICanonicalURL)
class WfTypedSharedContentCanonicalURL(WfSharedContentCanonicalURL):
    """Typed shared content canonical URL adapter"""

    def get_url(self, view_name=None, query=None):
        data_type = self.context.get_data_type()
        if data_type is not None:
            source = data_type.get_source_folder()
            if source is not None:
                return absolute_url(source, self.request,
                                    f"+/{ISequentialIdInfo(self.context).get_base_oid().strip()}"
                                    f"::{self.context.content_url}"
                                    f"{'/{}'.format(view_name) if view_name else '.html'}"
                                    f"{'?{}'.format(query) if query else ''}")
        return super().get_url(view_name, query)


@adapter_config(required=(ISharedContent, IPyAMSUserLayer),
                provides=ICanonicalURL)
class SharedContentCanonicalURL(ContextRequestAdapter):
    """Shared content canonical URL"""

    def get_url(self, display_context=None, view_name=None, query=None):
        """Canonical URL getter"""
        version = self.context.visible_version
        if version is not None:
            return canonical_url(version, self.request, view_name, query)
        return None


#
# Relative URL adapters
#

@adapter_config(required=(IWfSharedContent, IPyAMSUserLayer),
                provides=IRelativeURL)
class WfSharedContentRelativeURL(ContextRequestAdapter):
    """Workflow managed shared content relative URL adapter"""

    def get_url(self, display_context=None, view_name=None, query=None):
        """Relative URL getter"""
        query = get_quoted_query(query)
        return absolute_url(display_context, self.request,
                            f"+/{ISequentialIdInfo(self.context).get_base_oid().strip()}"
                            f"::{self.context.content_url}"
                            f"{'/{}'.format(view_name) if view_name else '.html'}"
                            f"{'?{}'.format(query) if query else ''}")


@adapter_config(required=(ISharedContent, IPyAMSUserLayer),
                provides=IRelativeURL)
class SharedContentRelativeURL(ContextRequestAdapter):
    """Shared content relative URL"""

    def get_url(self, display_context=None, view_name=None, query=None):
        """Relative URL getter"""
        version = self.context.visible_version
        if version is not None:
            return relative_url(version, self.request, display_context, view_name, query)
        return None



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.utils module

"""

from zope.interface import Interface


__docformat__ = 'restructuredtext'

from pyams_content.interfaces import IObjectType, IObjectTypes
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_utils.registry import get_current_registry


@adapter_config(required=Interface,
                provides=IObjectTypes)
class ObjectTypesAdapter(ContextAdapter):
    """Object_types adapter"""

    @property
    def object_types(self):
        result = set()
        registry = get_current_registry()
        for name, object_type in registry.getAdapters((self.context,), IObjectType):
            result.add(object_type)
        return result



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.glossary.task module

This module defines a custom scheduler task which can be used to handle
glossary automaton update.

This task has to be planned like all normal tasks.
"""

import sys
import traceback

from pyramid.events import subscriber
from zope.lifecycleevent import IObjectAddedEvent, IObjectModifiedEvent, IObjectRemovedEvent

from pyams_content.component.thesaurus import ITagsManager
from pyams_content.feature.glossary import get_glossary_automaton
from pyams_content.feature.glossary.interfaces import IGlossaryUpdaterTask
from pyams_scheduler.interfaces import IScheduler
from pyams_scheduler.interfaces.task import TASK_STATUS_FAIL, TASK_STATUS_OK
from pyams_scheduler.task import Task
from pyams_site.interfaces import ISiteRoot
from pyams_thesaurus.interfaces.term import IThesaurusTerm
from pyams_utils.factory import factory_config
from pyams_utils.registry import get_utility
from pyams_utils.request import check_request
from pyams_utils.traversing import get_parent


__docformat__ = 'restructuredtext'

from pyams_content import _


@factory_config(IGlossaryUpdaterTask)
class GlossaryUpdaterTask(Task):
    """Glossary updater task"""

    label = _("Glossary updater task")
    icon_class = 'fas fa-book'

    def run(self, report, **kwargs):  # pylint: disable=unused-argument
        """Run glossary automaton update"""
        try:
            report.write('Glossary automaton update\n'
                         '=========================\n')
            root = get_parent(self, ISiteRoot)
            automaton = get_glossary_automaton(root)
            report.write(f'Automaton size: {len(automaton)} terms\n')
            return TASK_STATUS_OK, automaton
        except Exception:  # pylint: disable=bare-except
            etype, value, tb = sys.exc_info()  # pylint: disable=invalid-name
            report.write('\n\n'
                         'An error occurred\n'
                         '=================\n')
            report.write(''.join(traceback.format_exception(etype, value, tb)))
            return TASK_STATUS_FAIL, None


@subscriber(IObjectAddedEvent, context_selector=IThesaurusTerm)
@subscriber(IObjectModifiedEvent, context_selector=IThesaurusTerm)
@subscriber(IObjectRemovedEvent, context_selector=IThesaurusTerm)
def handle_updated_thesaurus_term(event):
    """Reset glossary automaton on term update"""
    request = check_request()
    tags_manager = ITagsManager(request.root)
    if not tags_manager.enable_glossary:
        return
    scheduler = get_utility(IScheduler)
    for task in scheduler.values():
        if IGlossaryUpdaterTask.providedBy(task):
            task.launch()
            break



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.zmi.portal module

This modules defines custom components used for presentation templates management.
"""

from pyams_content.shared.common.interfaces import ISharedToolPortalContext, \
    IWfSharedContentPortalContext
from pyams_portal.zmi.presentation import PortalContextPresentationEditForm
from pyams_skin.interfaces.viewlet import IHelpViewletManager
from pyams_skin.viewlet.help import AlertMessage
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.interfaces import IAdminLayer


__docformat__ = 'restructuredtext'

from pyams_content import _


@viewlet_config(name='presentation-template.help',
                context=ISharedToolPortalContext, layer=IAdminLayer,
                view=PortalContextPresentationEditForm,
                manager=IHelpViewletManager, weight=10)
class SharedToolPortalContextPresentationEditFormHelp(AlertMessage):
    """Shared tool portal context presentation edit form help"""

    status = 'info'
    _message = _("You can select the default template which will be applied **by default** to "
                 "all contents inside this shared tool.<br />"
                 "If you select a shared template or choose to inherit from parent "
                 "configuration, you can adjust settings of each portlet but can't change "
                 "page configuration.<br />"
                 "If you choose to use a local template, it's configuration will only be "
                 "reusable in sub-levels which will choose to inherit from it.")

    message_renderer = 'markdown'


@viewlet_config(name='presentation-template.help',
                context=IWfSharedContentPortalContext, layer=IAdminLayer,
                view=PortalContextPresentationEditForm,
                manager=IHelpViewletManager, weight=10)
class SharedContextPortalContextPresentationEditFormHelp(AlertMessage):
    """Shared content portal context presentation edit form help"""

    status = 'info'
    _message = _("You can use the default template defined on the shared tool, which will "
                 "be applied to all contents.<br />"
                 "If you select another shared template or choose to inherit from this "
                 "configuration, you can adjust settings of each portlet but can't change "
                 "page configuration.<br />"
                 "If you choose to use a local template, it's configuration will not be "
                 "usable in other contents, except if you share it.")

    message_renderer = 'markdown'



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.skin.oid module

This module defines a custom route and a custom traverser used to support
'/+/{oid}::title.html' URL syntax support.
"""

from typing import Optional

from pyramid.httpexceptions import HTTPNotFound
from pyramid.interfaces import IRequest
from pyramid.response import Response
from pyramid.view import render_view_to_response, view_config
from zope.interface import Interface
from zope.traversing.interfaces import ITraversable

from pyams_content.interfaces import OID_ACCESS_ROUTE
from pyams_content.skin.interfaces import IPublicURL
from pyams_layer.interfaces import IPyAMSUserLayer
from pyams_sequence.interfaces import ISequentialIntIds
from pyams_sequence.reference import get_reference_target
from pyams_utils.adapter import ContextRequestAdapter, adapter_config
from pyams_utils.interfaces import DISPLAY_CONTEXT_KEY_NAME
from pyams_utils.registry import get_utility
from pyams_utils.url import absolute_url, canonical_url
from pyams_workflow.interfaces import IWorkflow, IWorkflowPublicationInfo, IWorkflowVersions

__docformat__ = 'restructuredtext'


def get_target(request: IRequest, oid: str) -> Optional[object]:
    """Get target from given reference"""
    sequence = get_utility(ISequentialIntIds)
    reference = sequence.get_full_oid(oid)
    target = get_reference_target(reference)
    if target is not None:
        workflow = IWorkflow(target, None)
        if workflow is not None:
            versions = IWorkflowVersions(target, None)
            if versions is not None:
                versions = versions.get_versions(workflow.visible_states, sort=True)
                if versions:
                    target = versions[-1]
    if (target is not None) and not IWorkflowPublicationInfo(target).is_visible(request):
        target = None
    return target


@view_config(route_name=OID_ACCESS_ROUTE)
def get_oid_access(request):
    """Get direct access to given OID

    This route can be used to get direct access to a given content,
    just by submitting an URL like /+/{oid}, where {oid} is the "short"
    sequence OID.
    """
    oid = request.matchdict.get('oid')
    if oid:
        oid = oid.split('::', 1)[0]
        view_name = ''.join(request.matchdict.get('view'))
        target = get_target(request, oid)
        if target is not None:
            if view_name:  # back-office access => last version
                public_url = request.registry.queryMultiAdapter((target, request), IPublicURL,
                                                                name=view_name)
                if public_url is not None:
                    location = public_url.get_url()
                else:
                    location = absolute_url(target, request, view_name)
            else:
                location = canonical_url(target, request, query=request.params)
            if location == request.url:
                # return view response to avoid infinite redirection!
                request.annotations[DISPLAY_CONTEXT_KEY_NAME] = request.context
                request.context = target
                response = render_view_to_response(target, request, view_name)
            else:
                response = Response()
                response.status_code = 302
                response.location = location
            return response
    raise HTTPNotFound()


@adapter_config(name='+',
                required=(Interface, IPyAMSUserLayer),
                provides=ITraversable)
@adapter_config(name='oid',
                required=(Interface, IPyAMSUserLayer),
                provides=ITraversable)
class OIDTraverser(ContextRequestAdapter):
    """++oid++ traverser

    This traverser can be used to get direct access to any content having an OID.
    The general URL syntax is "*/++oid++{oid}::{title}.html", where {oid} is the internal OID
    of the requested content, and "title" it's "content URL" attribute.

    A shorter syntax, is now available: */+/{oid}::{title}.html
    """

    def traverse(self, name, furtherpath=None):
        """++oid++ namespace traverser"""
        if not name:
            raise HTTPNotFound()
        context = self.context
        request = self.request
        if not hasattr(request, 'context'):
            request.context = context
        if '::' in name:
            oid, _title = name.split('::', 1)
        else:
            oid, _title = name, ''
        target = get_target(request, oid)
        if target is not None:
            request.annotations[DISPLAY_CONTEXT_KEY_NAME] = context
            request.context = target
            return target
        raise HTTPNotFound()



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.video.skin.provider.youtube module

This module defines YouTube external video renderer.
"""

from pyams_content.component.video.provider.interfaces import IYoutubeVideoSettings
from pyams_content.component.video.skin import IExternalVideoRenderer
from pyams_content.component.video.skin.provider import BaseExternalVideoRenderer, get_playlist_id, time_to_seconds
from pyams_layer.interfaces import IPyAMSLayer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config

__docformat__ = 'restructuredtext'


YOUTUBE_PARAMS = (
    ('start_at', 'start', time_to_seconds),
    ('stop_at', 'end', time_to_seconds),
    ('autoplay', 'autoplay', int),
    ('loop', 'loop', int),
    (None, 'playlist', get_playlist_id),
    ('show_commands', 'controls', int),
    ('hide_branding', 'modestbranding', int),
    ('show_related', 'rel', int),
    ('allow_fullscreen', 'fs', int),
    ('disable_keyboard', 'disablekb', int)
)


@adapter_config(required=(IYoutubeVideoSettings, IPyAMSLayer),
                provides=IExternalVideoRenderer)
@template_config(template='templates/youtube-render.pt', layer=IPyAMSLayer)
class YoutubeVideoRenderer(BaseExternalVideoRenderer):
    """Youtube video renderer"""

    params = YOUTUBE_PARAMS



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.gallery.portlet module

This module provides a portlet which can be used to display a medias gallery.
"""

from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.gallery import GalleryContainer
from pyams_content.component.gallery.portlet.interfaces import IGalleryPortletSettings
from pyams_portal.portlet import Portlet, PortletSettings, portlet_config
from pyams_utils.factory import factory_config


__docformat__ = 'restructuredtext'

from pyams_content import _


GALLERY_PORTLET_NAME = 'pyams_content.portlet.gallery'
GALLERY_ICON_CLASS = 'fas fa-images'


@factory_config(provided=IGalleryPortletSettings)
class GalleryPortletSettings(GalleryContainer, PortletSettings):
    """Gallery portlet settings"""

    title = FieldProperty(IGalleryPortletSettings['title'])
    use_context_gallery = FieldProperty(IGalleryPortletSettings['use_context_gallery'])


@portlet_config(permission=None)
class GalleryPortlet(Portlet):
    """Medias gallery portlet"""

    name = GALLERY_PORTLET_NAME
    label = _("Medias gallery")

    settings_factory = IGalleryPortletSettings
    toolbar_css_class = GALLERY_ICON_CLASS



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.site.zmi.manager module

This module provides site manager management interface components.
"""

from pyramid.events import subscriber
from zope.interface import Interface, Invalid

from pyams_content.interfaces import IBaseContent, MANAGE_SITE_TREE_PERMISSION
from pyams_content.root.zmi.sites import SiteRootSitesTable
from pyams_content.shared.common.interfaces import IBaseSharedTool, ISharedSite
from pyams_content.shared.site.interfaces import ISiteManager
from pyams_content.zmi.interfaces import IDashboardColumn, IDashboardContentType, \
    ISiteRootDashboardContentType
from pyams_content.zmi.properties import PropertiesEditForm
from pyams_form.ajax import AJAXFormRenderer, ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IAJAXFormRenderer, IDataExtractedEvent
from pyams_i18n.interfaces import II18n, INegotiator
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_site.interfaces import ISiteRoot
from pyams_skin.interfaces.viewlet import IBreadcrumbItem
from pyams_skin.viewlet.breadcrumb import BreadcrumbItem
from pyams_skin.viewlet.menu import MenuItem
from pyams_table.interfaces import ITable
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_utils.registry import get_utility
from pyams_utils.unicode import translate_string
from pyams_viewlet.manager import viewletmanager_config
from pyams_viewlet.viewlet import viewlet_config
from pyams_zmi.form import AdminModalAddForm
from pyams_zmi.helper.event import get_json_table_row_add_callback
from pyams_zmi.interfaces import IAdminLayer, IObjectLabel
from pyams_zmi.interfaces.table import ITableElementEditor
from pyams_zmi.interfaces.viewlet import IContextAddingsViewletManager, \
    IMenuHeader, IPropertiesMenu, ISiteManagementMenu
from pyams_zmi.table import TableElementEditor
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_content import _


@viewlet_config(name='add-site-manager.menu',
                context=ISiteRoot, layer=IAdminLayer, view=SiteRootSitesTable,
                manager=IContextAddingsViewletManager,
                permission=MANAGE_SITE_TREE_PERMISSION)
class SiteManagerAddMenu(MenuItem):
    """Site manager add menu"""

    label = _("Add site manager")
    icon_class = 'fas fa-sitemap'

    href = 'add-site-manager.html'
    modal_target = True


@ajax_form_config(name='add-site-manager.html',
                  context=ISiteRoot, layer=IPyAMSLayer,
                  permission=MANAGE_SITE_TREE_PERMISSION)
class SiteManagerAddForm(AdminModalAddForm):
    """Site manager add form"""

    title = _("Add site manager")
    legend = _("New site properties")

    fields = Fields(ISiteManager).select('title', 'short_name')
    content_factory = ISiteManager

    _edit_permission = MANAGE_SITE_TREE_PERMISSION

    def add(self, obj):
        short_name = II18n(obj).query_attribute('short_name', request=self.request)
        name = translate_string(short_name, force_lower=True, spaces='-')
        self.context[name] = obj


@subscriber(IDataExtractedEvent, form_selector=SiteManagerAddForm)
def handle_new_site_manager_data(event):
    """Handle new site manager data"""
    data = event.data
    negotiator = get_utility(INegotiator)
    name = data.get('short_name', {}).get(negotiator.server_language)
    if not name:
        event.form.widgets.errors += (Invalid(_("Site name is required!")),)
    else:
        name = translate_string(name, force_lower=True, spaces='-')
        if name in event.form.context:
            event.form.widgets.errors += (Invalid(_("A site manager is already registered with "
                                                    "this name!")),)


@adapter_config(required=(ISiteRoot, IAdminLayer, SiteManagerAddForm),
                provides=IAJAXFormRenderer)
class SiteManagerAddFormRenderer(ContextRequestViewAdapter):
    """Site manager add form renderer"""

    def render(self, changes):
        if changes is None:
            return None
        return {
            'callbacks': [
                get_json_table_row_add_callback(self.context, self.request,
                                                SiteRootSitesTable, changes)
            ]
        }


@adapter_config(required=(ISiteManager, IAdminLayer, Interface),
                provides=IObjectLabel)
def site_manager_label(context, request, view):
    """Site manager table element name"""
    return II18n(context).query_attribute('title', request=request)


@adapter_config(required=(ISiteManager, IAdminLayer, ITable),
                provides=ITableElementEditor)
class SiteManagerTableElementEditor(TableElementEditor):
    """Site manager table element editor"""

    view_name = 'admin'
    modal_target = False


@adapter_config(required=(ISiteManager, IAdminLayer, Interface),
                provides=IBreadcrumbItem)
class SiteManagerBreadcrumbs(BreadcrumbItem):
    """Site manager breadcrumb item"""

    @property
    def label(self):
        return II18n(self.context).query_attribute('short_name', request=self.request)

    view_name = 'admin'
    css_class = 'breadcrumb-item persistent strong'


@adapter_config(required=(ISiteManager, IAdminLayer, Interface, ISiteManagementMenu),
                provides=IMenuHeader)
def site_manager_management_menu_header(context, request, view, manager):
    """Site manager management menu header adapter"""
    return _("Site management")


@viewletmanager_config(name='properties.menu',
                       context=ISiteManager, layer=IAdminLayer,
                       manager=ISiteManagementMenu, weight=10,
                       provides=IPropertiesMenu,
                       permission=VIEW_SYSTEM_PERMISSION)
class SiteManagerPropertiesMenu(NavigationMenuItem):
    """Site manager properties menu"""

    label = _("Properties")
    icon_class = 'fas fa-edit'
    href = '#properties.html'


@ajax_form_config(name='properties.html',
                  context=ISiteManager, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class SiteManagerPropertiesEditForm(PropertiesEditForm):
    """Site manager properties edit form"""

    title = _("Site manager properties")
    legend = _("Main site properties")

    fields = Fields(ISiteManager).select('title', 'short_name', 'header', 'description') + \
        Fields(IBaseSharedTool).select('shared_content_workflow') + \
        Fields(ISiteManager).select('navigation_mode', 'notepad')


@adapter_config(required=(ISiteManager, IAdminLayer, SiteManagerPropertiesEditForm),
                provides=IAJAXFormRenderer)
class SiteManagerPropertiesEditFormRenderer(AJAXFormRenderer):
    """Site manager properties edit form renderer"""

    def render(self, changes):
        """AJAX form renderer"""
        if changes is None:
            return None
        if 'title' in changes.get(IBaseContent, ()):
            return {
                'status': 'reload',
                'message': self.request.localizer.translate(self.form.success_message)
            }
        return super().render(changes)


#
# Dashboards management adapters
#

@adapter_config(required=(ISharedSite, IAdminLayer, IDashboardColumn),
                provides=IDashboardContentType)
@adapter_config(required=(ISharedSite, IAdminLayer, IDashboardColumn),
                provides=ISiteRootDashboardContentType)
def site_manager_content_type(context, request, column):
    """Site manager content-type getter"""
    return request.localizer.translate(context.content_name)



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.search.portlet.skin.zmi module

"""

__docformat__ = 'restructuredtext'

from pyams_content.feature.search.portlet.skin import ISearchResultsPortletBaseRendererSettings, \
    ISearchResultsPortletCardsRendererSettings, ISearchResultsPortletPanelsRendererSettings
from pyams_form.field import Fields
from pyams_form.group import Group
from pyams_form.interfaces.form import IFormFields, IGroup
from pyams_portal.zmi.interfaces import IPortletRendererSettingsEditForm
from pyams_utils.adapter import adapter_config
from pyams_zmi.form import FormGroupChecker
from pyams_zmi.interfaces import IAdminLayer

from pyams_content import _


#
# Base search results renderer settings
#

@adapter_config(required=(ISearchResultsPortletBaseRendererSettings, IAdminLayer, IPortletRendererSettingsEditForm),
                provides=IFormFields)
def search_results_portlet_renderer_settings_fields(context, request, form):
    """Search results portlet renderer settings fields getter"""
    return Fields(ISearchResultsPortletBaseRendererSettings).select(
        'display_if_empty', 'display_results_count', 'allow_sorting', 'allow_pagination')


@adapter_config(name='illustration',
                required=(ISearchResultsPortletBaseRendererSettings, IAdminLayer, IPortletRendererSettingsEditForm),
                provides=IGroup)
class SearchResultsPortletRendererIllustrationSettingsGroup(FormGroupChecker):
    """Search results portlet renderer illustration settings group"""

    fields = Fields(ISearchResultsPortletBaseRendererSettings).select(
        'display_illustrations', 'thumb_selection')
    weight = 10


@adapter_config(name='illustration',
                required=(ISearchResultsPortletPanelsRendererSettings, IAdminLayer, IPortletRendererSettingsEditForm),
                provides=IGroup)
class SearchResultsPortletPanelsRendererIllustrationSettingsGroup(FormGroupChecker):
    """Search results portlet panels renderer illustration settings group"""

    fields = Fields(ISearchResultsPortletPanelsRendererSettings).select(
        'display_illustrations', 'thumb_selection')
    weight = 10


@adapter_config(name='illustration',
                required=(ISearchResultsPortletCardsRendererSettings, IAdminLayer, IPortletRendererSettingsEditForm),
                provides=IGroup)
class SearchResultsPortletCardsRendererIllustrationSettingsGroup(FormGroupChecker):
    """Search results portlet cards renderer illustration settings group"""

    fields = Fields(ISearchResultsPortletCardsRendererSettings).select(
        'display_illustrations', 'thumb_selection')
    weight = 10


@adapter_config(name='header',
                required=(ISearchResultsPortletBaseRendererSettings, IAdminLayer, IPortletRendererSettingsEditForm),
                provides=IGroup)
class SEarchResultsPortletRendererHeaderSettingsGroup(Group):
    """Search results portlet renderer header settings group"""

    legend = _("Header display")

    fields = Fields(ISearchResultsPortletBaseRendererSettings).select(
        'header_display_mode', 'start_length')
    weight = 20


@adapter_config(name='button',
                required=(ISearchResultsPortletCardsRendererSettings, IAdminLayer, IPortletRendererSettingsEditForm),
                provides=IGroup)
class ViewItemsPanelsRendererButtonSettingsGroup(Group):
    """View item portlet panels renderer button settings group"""

    fields = Fields(ISearchResultsPortletCardsRendererSettings).select('button_title')
    weight = 30



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.gallery module

This module defines persistent components and adapters used to handle medias
galleries.
"""

from persistent import Persistent
from zope.container.contained import Contained
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.video.interfaces import IExternalVideo, IExternalVideoSettings, IExternalVideoProvider
from pyams_utils.adapter import adapter_config, get_annotation_adapter
from pyams_utils.factory import factory_config
from pyams_utils.registry import get_utility, query_utility


@factory_config(IExternalVideo)
class ExternalVideo(Persistent, Contained):
    """External video persistent class"""

    author = FieldProperty(IExternalVideo['author'])
    description = FieldProperty(IExternalVideo['description'])
    provider_name = FieldProperty(IExternalVideo['provider_name'])

    def get_provider(self):
        return query_utility(IExternalVideoProvider, name=self.provider_name or '')

    @property
    def settings(self):
        provider = self.get_provider()
        if provider is None:
            return None
        return provider.settings_interface(self)


EXTERNAL_VIDEO_SETTINGS_KEY = 'pyams_content.video::{}'


@adapter_config(required=IExternalVideo,
                provides=IExternalVideoSettings)
def external_video_settings(context, provider_name=None):
    """External video settings factory"""
    if not provider_name:
        provider_name = context.provider_name
        if not provider_name:
            return None
    provider = get_utility(IExternalVideoProvider, name=provider_name)
    settings_key = EXTERNAL_VIDEO_SETTINGS_KEY.format(provider_name.lower())
    return get_annotation_adapter(context, settings_key,
                                  factory=IExternalVideoSettings(provider))



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

__docformat__ = 'restructuredtext'

from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.paragraph import IBaseParagraph, IParagraphContainer, \
    IParagraphContainerTarget
from pyams_content.component.paragraph.portlet.interfaces import \
    IParagraphContainerPortletSettings, IParagraphNavigationPortletSettings
from pyams_portal.portlet import Portlet, PortletSettings, portlet_config
from pyams_security.interfaces.base import VIEW_PERMISSION
from pyams_sequence.reference import InternalReferenceMixin
from pyams_utils.factory import factory_config, get_object_factory
from pyams_utils.interfaces import MISSING_INFO
from pyams_utils.request import check_request
from pyams_utils.traversing import get_parent
from pyams_zmi.utils import get_object_label

from pyams_content import _


class ParagraphPortletSettingsMixin:
    """Paragraph container portlet settings mixin class"""

    def get_paragraphs_labels(self):
        """Paragraphs labels getter"""
        if not self.paragraphs:
            yield MISSING_INFO
        else:
            target = get_parent(self, IParagraphContainerTarget)
            if target is not None:
                container = IParagraphContainer(target)
                request = check_request()
                for name in self.paragraphs:
                    paragraph = container.get(name)
                    if name is not None:
                        yield get_object_label(paragraph, request)

    def get_paragraph_types_labels(self):
        """Paragraphs types labels getter"""
        if not self.factories:
            yield MISSING_INFO
        else:
            request = check_request()
            for factory_name in self.factories:
                factory = get_object_factory(IBaseParagraph, name=factory_name)
                if factory is not None:
                    yield request.localizer.translate(factory.factory.factory_name)


#
# Paragraph container portlet
#

PARAGRAPH_CONTAINER_PORTLET_NAME = 'pyams_content.portlet.paragraphs'


@factory_config(IParagraphContainerPortletSettings)
class ParagraphContainerPortletSettings(ParagraphPortletSettingsMixin, InternalReferenceMixin,
                                        PortletSettings):
    """Paragraph container portlet settings"""

    title = FieldProperty(IParagraphContainerPortletSettings['title'])
    button_label = FieldProperty(IParagraphContainerPortletSettings['button_label'])
    paragraphs = FieldProperty(IParagraphContainerPortletSettings['paragraphs'])
    factories = FieldProperty(IParagraphContainerPortletSettings['factories'])
    anchors_only = FieldProperty(IParagraphContainerPortletSettings['anchors_only'])
    exclude_anchors = FieldProperty(IParagraphContainerPortletSettings['exclude_anchors'])
    display_navigation_links = FieldProperty(
        IParagraphContainerPortletSettings['display_navigation_links'])
    limit = FieldProperty(IParagraphContainerPortletSettings['limit'])


@portlet_config(permission=VIEW_PERMISSION)
class ParagraphContainerPortlet(Portlet):
    """Paragraphs container portlet"""

    name = PARAGRAPH_CONTAINER_PORTLET_NAME
    label = _("Content paragraphs")

    settings_factory = IParagraphContainerPortletSettings
    toolbar_css_class = 'fas fa-paragraph'


#
# Paragraph navigation portlet
#

PARAGRAPH_NAVIGATION_PORTLET_NAME = 'pyams_content.portlet.paragraphs.navigation'


@factory_config(IParagraphNavigationPortletSettings)
class ParagraphNavigationPortletSettings(ParagraphPortletSettingsMixin, PortletSettings):
    """Paragraph navigation portlet settings"""

    paragraphs = FieldProperty(IParagraphNavigationPortletSettings['paragraphs'])
    factories = FieldProperty(IParagraphNavigationPortletSettings['factories'])
    anchors_only = FieldProperty(IParagraphNavigationPortletSettings['anchors_only'])


@portlet_config(permission=VIEW_PERMISSION)
class ParagraphNavigationPortlet(Portlet):
    """Paragraphs navigation portlet"""

    name = PARAGRAPH_NAVIGATION_PORTLET_NAME
    label = _("Paragraphs navigation anchors")

    settings_factory = IParagraphNavigationPortletSettings
    toolbar_css_class = 'fas fa-angle-double-down'



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.interfaces.types module

This module defines interfaces related to content data types.
"""

from zope.container.constraints import contains
from zope.container.interfaces import IContainer
from zope.interface import Attribute
from zope.location import ILocation
from zope.schema import Bool, Choice, List

from pyams_content.reference.pictogram import PICTOGRAM_VOCABULARY
from pyams_content.shared.common import IBaseSharedTool
from pyams_content.shared.common.interfaces import IBaseContentPortalContext, IWfSharedContent
from pyams_i18n.schema import I18nTextLineField
from pyams_portal.interfaces import IPortalContext
from pyams_sequence.schema import InternalReferenceField

__docformat__ = 'restructuredtext'

from pyams_content import _
from pyams_utils.schema import ColorField

DATA_TYPES_VOCABULARY = 'pyams_content.datatypes'
"""Vocabulary of shared content data types"""

VISIBLE_DATA_TYPES_VOCABULARY = 'pyams_content.datatypes.visible'
"""Vocabulary of shared content visible data types"""

ALL_DATA_TYPES_VOCABULARY = 'pyams_content.datatypes.all'
"""Vocabulary of all shared content tools data types"""

DATA_TYPE_FIELDS_VOCABULARY = 'pyams_content.datatype.fields'
"""Vocabulary of data type fields"""


class IDataType(ILocation):
    """Data interface for data-types"""

    visible = Bool(title=_("Visible?"),
                   description=_("An hidden data type can't be assigned to new contents"),
                   required=True,
                   default=True)

    label = I18nTextLineField(title=_("Label"),
                              required=True)

    source_folder = InternalReferenceField(title=_("Source folder"),
                                           description=_("Source folder (or search engine) to "
                                                         "which content is attached; if this "
                                                         "reference is set, canonical URL will "
                                                         "be based on this object's URL"),
                                           required=False)

    def get_source_folder(self):
        """Return source folder as object"""

    navigation_label = I18nTextLineField(title=_("Navigation label"),
                                         description=_("Label used for navigation entries"),
                                         required=False)

    backoffice_label = I18nTextLineField(title=_("Back-office label"),
                                         description=_("Optional label used in management pages "
                                                       "instead of default label"),
                                         required=False)

    color = ColorField(title=_("Color"),
                       description=_("Color used to illustrate this data type"),
                       required=False)

    pictogram = Choice(title=_("Pictogram"),
                       description=_("Pictogram associated with this data type"),
                       vocabulary=PICTOGRAM_VOCABULARY,
                       required=False)

    pictogram_on = Choice(title=_("ON pictogram"),
                          description=_("'On' state pictogram associated with this data type"),
                          vocabulary=PICTOGRAM_VOCABULARY,
                          required=False)

    pictogram_off = Choice(title=_("OFF pictogram"),
                           description=_("'Off' state pictogram associated with this data type"),
                           vocabulary=PICTOGRAM_VOCABULARY,
                           required=False)

    display_as_tag = Bool(title=_("Display as tag?"),
                          description=_("Some portlets renderers can display a small tag above "
                                        "each content to show their content type; if this option "
                                        "is checked, data type label will be displayed instead "
                                        "of content type"),
                          required=True,
                          default=False)

    field_names = List(title=_("Field names"),
                       description=_("List of fields associated with this data type"),
                       value_type=Choice(vocabulary=DATA_TYPE_FIELDS_VOCABULARY))


#
# Types data manager interfaces
#

DATA_MANAGER_ANNOTATION_KEY = 'pyams_content.types.manager'


class ITypedDataManager(IContainer):
    """Typed shared data manager interface"""

    contains(IDataType)

    def get_visible_items(self):
        """Iterator on visible data types"""


class ITypedSharedTool(IBaseSharedTool):
    """Shared tool containing typed data"""

    shared_content_info_factory = Attribute("Custom shared content information factory")


class ITypedSharedToolPortalContext(ITypedSharedTool, IPortalContext):
    """Typed shared tool with portal context"""


#
# Typed content interfaces
#

class IWfTypedSharedContent(IWfSharedContent):
    """Typed shared content"""

    data_type = Choice(title=_("Data type"),
                       description=_("Type of content data"),
                       required=True,
                       vocabulary=VISIBLE_DATA_TYPES_VOCABULARY)

    def get_data_type(self):
        """Get associated data type"""

    field_names = Attribute("Selected data type field names")


class IWfTypedSharedContentPortalContext(IWfTypedSharedContent, IBaseContentPortalContext):
    """Shared content with portal support"""



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.shared.common.zmi.search module

This module provides components used for contents searching from dashboards.
"""

from hypatia.catalog import CatalogQuery
from hypatia.interfaces import ICatalog
from hypatia.query import And, Any, Contains, Eq, Ge, Le
from pyramid.interfaces import IView
from pyramid.view import view_config
from zope.dublincore.interfaces import IZopeDublinCore
from zope.interface import Interface, implementer
from zope.intid import IIntIds
from zope.schema import Choice, TextLine
from zope.schema.vocabulary import getVocabularyRegistry

from pyams_catalog.query import CatalogResultSet
from pyams_content.component.thesaurus import ICollectionsManager, ITagsManager, IThemesManager
from pyams_content.shared.common import IBaseSharedTool, SHARED_CONTENT_TYPES_VOCABULARY
from pyams_content.shared.common.interfaces import SHARED_TOOL_WORKFLOW_STATES_VOCABULARY
from pyams_content.shared.common.interfaces.types import DATA_TYPES_VOCABULARY
from pyams_content.shared.common.zmi.dashboard import BaseSharedToolDashboardSingleView, \
    DashboardTable, BaseSharedToolDashboardView, SharedToolDashboardView
from pyams_content.zmi.interfaces import IAllDashboardMenu
from pyams_form.field import Fields
from pyams_form.interfaces.form import IFormFields, IGroup
from pyams_i18n.interfaces import INegotiator
from pyams_layer.interfaces import IPyAMSLayer
from pyams_layer.skin import apply_skin
from pyams_pagelet.pagelet import pagelet_config
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_security.schema import PrincipalField
from pyams_sequence.interfaces import ISequentialIntIds
from pyams_sequence.workflow import get_last_version
from pyams_skin.interfaces.viewlet import IHeaderViewletManager
from pyams_table.interfaces import IValues
from pyams_template.template import template_config
from pyams_thesaurus.schema import ThesaurusTermsListField
from pyams_thesaurus.zmi.widget import ThesaurusTermsTreeFieldWidget
from pyams_utils.adapter import ContextRequestViewAdapter, NullAdapter, adapter_config
from pyams_utils.list import unique_iter
from pyams_utils.registry import get_utility
from pyams_utils.schema import DatetimesRangeField
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_utils.vocabulary import vocabulary_config
from pyams_viewlet.viewlet import EmptyViewlet, ViewContentProvider, viewlet_config
from pyams_workflow.interfaces import IWorkflow, IWorkflowVersions
from pyams_zmi.form import FormGroupSwitcher
from pyams_zmi.interfaces import IAdminLayer
from pyams_zmi.interfaces.table import IInnerTable
from pyams_zmi.search import SearchForm, SearchResultsView, SearchView
from pyams_zmi.skin import AdminSkin
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem

__docformat__ = 'restructuredtext'

from pyams_content import _


@implementer(IView)
class SharedToolQuickSearchResultsTable(DashboardTable):
    """Shared tool quick search results table"""


def get_quick_search_params(query, request, catalog, sequence):
    """Get quick search params"""
    query_params = Eq(catalog['oid'], sequence.get_full_oid(query))
    negotiator = get_utility(INegotiator)
    for lang in {
                    request.registry.settings.get('pyramid.default_locale_name', 'en'),
                    request.locale_name,
                    negotiator.server_language
                } | negotiator.offered_languages:
        index_name = f'title:{lang}'
        if index_name in catalog:
            index = catalog[index_name]
            if index.check_query(query):
                query_params |= Contains(index,
                                         ' and '.join((f'{w}*' for w in query.split())))
    return query_params


@adapter_config(required=(IBaseSharedTool, IPyAMSLayer, SharedToolQuickSearchResultsTable),
                provides=IValues)
class SharedToolQuickSearchResultsValues(ContextRequestViewAdapter):
    """Shared tool quick search results values adapter"""

    @property
    def values(self):
        """Table values getter"""
        intids = get_utility(IIntIds)
        catalog = get_utility(ICatalog)
        query = self.request.params.get('query', '').strip()
        if not query:
            return ()
        sequence = get_utility(ISequentialIntIds)
        query = query.lower().replace('*', '')
        if query.startswith('+'):
            params = Eq(catalog['oid'], sequence.get_full_oid(query))
        else:
            vocabulary = getVocabularyRegistry().get(self.context,
                                                     SHARED_CONTENT_TYPES_VOCABULARY)
            params = And(Eq(catalog['parents'], intids.register(self.context)),
                         Any(catalog['content_type'], vocabulary.by_value.keys()))
            params &= get_quick_search_params(query, self.request, catalog, sequence)
        yield from unique_iter(map(get_last_version,
                                   CatalogResultSet(CatalogQuery(catalog).query(
                                       params, sort_index='modified_date', reverse=True))))


def get_json_search_results(request, results):
    """Get search results as JSON response"""
    if len(results.values) == 1:
        result = results.values[0]
        return {
            'status': 'redirect',
            'location': absolute_url(result, request, 'admin')
        }
    apply_skin(request, AdminSkin)
    results.update()
    return {
        'status': 'info',
        'content': {
            'html': results.render()
        }
    }


@view_config(name='quick-search.json',
             context=IBaseSharedTool, request_type=IPyAMSLayer,
             permission=VIEW_SYSTEM_PERMISSION, renderer='json', xhr=True)
def shared_tool_quick_search_view(request):
    """Shared tool quick search view"""
    results = SharedToolQuickSearchResultsTable(request.context, request)
    return get_json_search_results(request, results)


@adapter_config(name='quick-search',
                required=(IBaseSharedTool, IAdminLayer, SharedToolDashboardView),
                provides=IInnerTable)
@template_config(template='templates/quick-search.pt', layer=IAdminLayer)
class SharedToolQuickSearchView(BaseSharedToolDashboardView):
    """Shared tool quick search view"""

    @property
    def legend(self):
        """Legend getter"""
        translate = self.request.localizer.translate
        return translate(_("Between all contents of « {} » content type")) \
            .format(translate(self.context.shared_content_factory.factory.content_name))

    def render(self):
        """Viewlet renderer"""
        return ViewContentProvider.render(self)


@pagelet_config(name='quick-search.html',
                context=IBaseSharedTool, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class SharedToolQuickSearchPageView(BaseSharedToolDashboardSingleView):
    """Shared tool quick search page view"""

    table_class = SharedToolQuickSearchResultsTable

    empty_label = _("SEARCH - No result found")
    single_label = _("SEARCH - 1 result found")
    plural_label = _("SEARCH - {} results found")


#
# Advanced search components
#

@vocabulary_config(name=SHARED_TOOL_WORKFLOW_STATES_VOCABULARY)
def WorkflowStatesVocabulary(context):
    """Workflow states vocabulary"""
    target = get_parent(context, IBaseSharedTool)
    if target is not None:
        workflow = IWorkflow(target)
        return workflow.states


@viewlet_config(name='advanced-search.menu',
                context=IBaseSharedTool, layer=IAdminLayer,
                manager=IAllDashboardMenu, weight=40,
                permission=VIEW_SYSTEM_PERMISSION)
class SharedToolAdvancedSearchMenu(NavigationMenuItem):
    """Shared tool advanced search menu"""

    label = _("Advanced search")
    href = "#advanced-search.html"


class ISharedToolAdvancedSearchQuery(Interface):
    """Shared tool advanced search query interface"""

    query = TextLine(title=_("Search text"),
                     description=_("Entered text will be search in title only"),
                     required=False)

    owner = PrincipalField(title=_("Owner"),
                           required=False)

    status = Choice(title=_("Status"),
                    vocabulary=SHARED_TOOL_WORKFLOW_STATES_VOCABULARY,
                    required=False)

    data_type = Choice(title=_("Data type"),
                       vocabulary=DATA_TYPES_VOCABULARY,
                       required=False)

    created = DatetimesRangeField(title=_("Creation date"),
                                  required=False)

    modified = DatetimesRangeField(title=_("Modification date"),
                                   required=False)

    tags = ThesaurusTermsListField(title=_("Tags"),
                                   required=False)

    themes = ThesaurusTermsListField(title=_("Themes"),
                                     required=False)

    collections = ThesaurusTermsListField(title=_("Collections"),
                                          required=False)


class SharedToolAdvancedSearchForm(SearchForm):
    """Shared tool advanced search form"""

    title = _("Contents search form")

    ajax_form_handler = 'advanced-search-results.html'
    _edit_permission = VIEW_SYSTEM_PERMISSION


@adapter_config(required=(Interface, IAdminLayer, SharedToolAdvancedSearchForm),
                provides=IFormFields)
def shared_tool_advanced_search_form_fields(context, request, form):
    """Shared tool advanced search form fields getter"""
    return Fields(ISharedToolAdvancedSearchQuery).omit('tags', 'themes', 'collections')


class BaseThesaurusTermsSearchGroup(FormGroupSwitcher):
    """Base thesaurus terms search group"""

    fieldname = None
    manager = None

    def __new__(cls, context, request, view):
        manager = cls.manager(request.root)
        if not manager.thesaurus_name:
            return None
        return FormGroupSwitcher.__new__(cls)

    @property
    def fields(self):
        """Fields getter"""
        fields = Fields(ISharedToolAdvancedSearchQuery).select(self.fieldname)
        fields[self.fieldname].widget_factory = ThesaurusTermsTreeFieldWidget
        return fields

    label_css_class = 'hidden'
    input_css_class = 'col-12'

    def update_widgets(self, prefix=None, use_form_mode=True):
        super().update_widgets(prefix, use_form_mode)
        terms = self.widgets.get(self.fieldname)
        if terms is not None:
            manager = self.manager(self.request.root)
            terms.thesaurus_name = manager.thesaurus_name
            terms.extract_name = manager.extract_name


@adapter_config(name='tags',
                required=(IBaseSharedTool, IAdminLayer, SharedToolAdvancedSearchForm),
                provides=IGroup)
class SharedToolAdvancedSearchFormTagsGroup(BaseThesaurusTermsSearchGroup):
    """Shared tool advanced search form tags group"""

    legend = _("Tags")

    fieldname = 'tags'
    manager = ITagsManager

    weight = 10


@adapter_config(name='themes',
                required=(IBaseSharedTool, IAdminLayer, SharedToolAdvancedSearchForm),
                provides=IGroup)
class SharedToolAdvancedSearchFormThemesGroup(BaseThesaurusTermsSearchGroup):
    """Shared tool advanced search form themes group"""

    legend = _("Themes")

    fieldname = 'themes'
    manager = IThemesManager

    weight = 20


@adapter_config(name='collections',
                required=(IBaseSharedTool, IAdminLayer, SharedToolAdvancedSearchForm),
                provides=IGroup)
class SharedToolAdvancedSearchFormCollectionsGroup(BaseThesaurusTermsSearchGroup):
    """Shared tool advanced search form collections group"""

    legend = _("Collections")

    fieldname = 'collections'
    manager = ICollectionsManager

    weight = 30


@pagelet_config(name='advanced-search.html',
                context=IBaseSharedTool, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION)
class SharedToolAdvancedSearchView(SearchView):
    """Shared tool advanced search view"""

    title = _("Contents search form")
    header_label = _("Advanced search")
    search_form = SharedToolAdvancedSearchForm


class SharedToolAdvancedSearchResultsTable(DashboardTable):
    """Shared tool advanced search results table"""


@adapter_config(required=(IBaseSharedTool, IPyAMSLayer, SharedToolAdvancedSearchResultsTable),
                provides=IValues)
class SharedToolAdvancedSearchResultsValues(ContextRequestViewAdapter):
    """Shared tool advanced search results values"""

    def get_params(self, data):
        """Extract catalog query params from incoming request"""
        intids = get_utility(IIntIds)
        catalog = get_utility(ICatalog)
        vocabulary = getVocabularyRegistry().get(self.context, SHARED_CONTENT_TYPES_VOCABULARY)
        params = And(Eq(catalog['parents'], intids.register(self.context)),
                     Any(catalog['content_type'], vocabulary.by_value.keys()))
        query = data.get('query')
        if query:
            sequence = get_utility(ISequentialIntIds)
            if query.startswith('+'):
                params &= Eq(catalog['oid'], sequence.get_full_oid(query))
            else:
                query_params = Eq(catalog['oid'], sequence.get_full_oid(query))
                negotiator = get_utility(INegotiator)
                for lang in {self.request.registry.settings.get('pyramid.default_locale_name',
                                                                'en'),
                             self.request.locale_name,
                             negotiator.server_language} | negotiator.offered_languages:
                    index_name = 'title:{0}'.format(lang)
                    if index_name in catalog:
                        index = catalog[index_name]
                        if index.check_query(query):
                            query_params |= Contains(index,
                                                     ' and '.join((w+'*' for w in query.split())))
                params &= query_params
        if data.get('owner'):
            params &= Eq(catalog['role:owner'], data['owner'])
        if data.get('status'):
            params &= Eq(catalog['workflow_state'], data['status'])
        if data.get('data_type'):
            params &= Eq(catalog['data_type'], data['data_type'])
        created_after, created_before = data.get('created', (None, None))
        if created_after:
            params &= Ge(catalog['created_date'], created_after)
        if created_before:
            params &= Le(catalog['created_date'], created_before)
        modified_after, modified_before = data.get('modified', (None, None))
        if modified_after:
            params &= Ge(catalog['modified_date'], modified_after)
        if modified_before:
            params &= Le(catalog['modified_date'], modified_before)
        if data.get('tags'):
            tags = [intids.register(term) for term in data['tags']]
            params &= Any(catalog['tags'], tags)
        if data.get('themes'):
            tags = [intids.register(term) for term in data['themes']]
            params &= Any(catalog['themes'], tags)
        if data.get('collections'):
            tags = [intids.register(term) for term in data['collections']]
            params &= Any(catalog['collections'], tags)
        return params

    @property
    def values(self):
        """Shared tool advanced search results values getter"""
        form = SharedToolAdvancedSearchForm(self.context, self.request)
        form.update()
        data, _errors = form.extract_data()
        params = self.get_params(data)
        catalog = get_utility(ICatalog)
        if data.get('status'):
            yield from unique_iter(
                map(lambda x: sorted(IWorkflowVersions(x).get_versions(data['status']),
                                     key=lambda y: IZopeDublinCore(y).modified)[0],
                    CatalogResultSet(CatalogQuery(catalog).query(
                        params, sort_index='modified_date', reverse=True))))
        else:
            yield from unique_iter(
                map(get_last_version,
                    CatalogResultSet(CatalogQuery(catalog).query(
                        params, sort_index='modified_date', reverse=True))))


@pagelet_config(name='advanced-search-results.html',
                context=IBaseSharedTool, layer=IPyAMSLayer,
                permission=VIEW_SYSTEM_PERMISSION, xhr=True)
class SharedToolAdvancedSearchResultsView(SearchResultsView):
    """Shared tool advanced search results view"""

    table_label = _("Search results")
    table_class = SharedToolAdvancedSearchResultsTable


@viewlet_config(name='pyams.content_header',
                layer=IAdminLayer, view=SharedToolAdvancedSearchResultsView,
                manager=IHeaderViewletManager, weight=10)
class AdvancedSearchResultsViewHeaderViewlet(EmptyViewlet):
    """Advanced search results view header viewlet"""

    def render(self):
        return '<h1 class="mt-3"></h1>'



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.root.zmi module

Site root management components.
"""

from zope.interface import Interface

from pyams_content.root import ISiteRootInfos
from pyams_content.zmi.properties import PropertiesEditForm
from pyams_form.ajax import ajax_form_config
from pyams_form.field import Fields
from pyams_form.interfaces.form import IAJAXFormRenderer, IFormContent
from pyams_layer.interfaces import IPyAMSLayer
from pyams_security.interfaces.base import VIEW_SYSTEM_PERMISSION
from pyams_site.interfaces import ISiteRoot
from pyams_utils.adapter import ContextRequestViewAdapter, adapter_config
from pyams_viewlet.manager import viewletmanager_config
from pyams_zmi.interfaces import IAdminLayer, IObjectLabel
from pyams_zmi.interfaces.configuration import IZMIConfiguration
from pyams_zmi.interfaces.viewlet import IMenuHeader, IPropertiesMenu, ISiteManagementMenu
from pyams_zmi.zmi.viewlet.menu import NavigationMenuItem


__docformat__ = 'restructuredtext'

from pyams_content import _


@adapter_config(required=(ISiteRoot, IAdminLayer, Interface, ISiteManagementMenu),
                provides=IMenuHeader)
def site_root_management_menu_header(context, request, view, manager):
    """Site root management menu header adapter"""
    return _("Main site management")


@adapter_config(required=(ISiteRoot, IAdminLayer, Interface),
                provides=IObjectLabel)
def site_root_label(context, request, view):
    """Site root label"""
    return IZMIConfiguration(request.root).site_name


@viewletmanager_config(name='properties.menu',
                       context=ISiteRoot, layer=IAdminLayer,
                       manager=ISiteManagementMenu, weight=10,
                       provides=IPropertiesMenu,
                       permission=VIEW_SYSTEM_PERMISSION)
class SiteRootPropertiesMenu(NavigationMenuItem):
    """Site root properties menu"""

    label = _("Properties")
    icon_class = 'fas fa-edit'
    href = '#properties.html'


@ajax_form_config(name='properties.html',
                  context=ISiteRoot, layer=IPyAMSLayer,
                  permission=VIEW_SYSTEM_PERMISSION)
class SiteRootPropertiesEditForm(PropertiesEditForm):
    """Site root properties edit form"""

    title = _("Main site properties")
    legend = _("Site information")

    fields = Fields(ISiteRootInfos)

    def update_widgets(self, prefix=None):
        super().update_widgets(prefix)
        description = self.widgets.get('description')
        if description is not None:
            description.set_widgets_attr('rows', 5)


@adapter_config(required=(ISiteRoot, IPyAMSLayer, SiteRootPropertiesEditForm),
                provides=IFormContent)
def site_root_properties_form_content(context, request, form):
    """Site root properties edit form content getter"""
    return ISiteRootInfos(context)


@adapter_config(required=(ISiteRoot, IAdminLayer, SiteRootPropertiesEditForm),
                provides=IAJAXFormRenderer)
class SiteRootPropertiesEditFormRenderer(ContextRequestViewAdapter):
    """Site root properties edit form renderer"""

    def render(self, changes):
        """Form renderer"""
        if changes is None:
            return None
        return {
            'status': 'reload',
            'smallbox': {
                'status': 'success',
                'message': self.request.localizer.translate(self.view.success_message)
            }
        }



#
# Copyright (c) 2015-2021 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.component.links module

"""

from html import escape

from pyramid.encode import url_quote, urlencode
from pyramid.events import subscriber
from zope.interface import alsoProvides, directlyProvidedBy, implementer, noLongerProvides
from zope.lifecycleevent import IObjectAddedEvent, IObjectModifiedEvent
from zope.schema.fieldproperty import FieldProperty
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary

from pyams_content.component.association import AssociationItem
from pyams_content.component.association.interfaces import IAssociationContainer, \
    IAssociationContainerTarget, IAssociationInfo
from pyams_content.component.links.interfaces import CONTENT_LINKS_VOCABULARY, \
    EXTERNAL_LINK_ICON_CLASS, EXTERNAL_LINK_ICON_HINT, IBaseLink, \
    IExternalLink, IInternalLink, IInternalLinkCustomInfo, IInternalLinkCustomInfoTarget, \
    IMailtoLink, INTERNAL_LINK_ICON_CLASS, INTERNAL_LINK_ICON_HINT, MAILTO_LINK_ICON_CLASS, \
    MAILTO_LINK_ICON_HINT
from pyams_content.reference.pictogram.interfaces import IPictogramTable
from pyams_i18n.interfaces import II18n
from pyams_sequence.interfaces import IInternalReference, ISequentialIdInfo
from pyams_sequence.reference import InternalReferenceMixin
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_utils.factory import factory_config
from pyams_utils.interfaces import MISSING_INFO
from pyams_utils.registry import query_utility
from pyams_utils.request import check_request
from pyams_utils.traversing import get_parent
from pyams_utils.url import canonical_url, relative_url
from pyams_utils.vocabulary import vocabulary_config
from pyams_utils.zodb import volatile_property
from pyams_workflow.interfaces import IWorkflowPublicationInfo


__docformat__ = 'restructuredtext'


#
# Links vocabulary
#

@vocabulary_config(name=CONTENT_LINKS_VOCABULARY)
class ContentLinksVocabulary(SimpleVocabulary):
    """Content links vocabulary"""

    def __init__(self, context=None):
        terms = []
        target = get_parent(context, IAssociationContainerTarget)
        if target is not None:
            terms = [
                SimpleTerm(link.__name__, title=IAssociationInfo(link).inner_title)
                for link in IAssociationContainer(target).values()
                if IBaseLink.providedBy(link)
            ]
        super(ContentLinksVocabulary, self).__init__(terms)


#
# Base link persistent class
#

@implementer(IBaseLink)
class BaseLink(AssociationItem):
    """Base link persistent class"""

    title = FieldProperty(IBaseLink['title'])
    description = FieldProperty(IBaseLink['description'])
    _pictogram_name = FieldProperty(IBaseLink['pictogram_name'])

    @property
    def pictogram_name(self):
        """Pictogram name getter"""
        return self._pictogram_name

    @pictogram_name.setter
    def pictogram_name(self, value):
        """Pictogram name setter"""
        if value != self._pictogram_name:
            self._pictogram_name = value
            del self.pictogram

    @volatile_property
    def pictogram(self):
        """Pictogram getter"""
        table = query_utility(IPictogramTable)
        if table is not None:
            return table.get(self._pictogram_name)


class BaseLinkInfoAdapter(ContextAdapter):
    """Base link association info adapter"""

    @property
    def pictogram(self):
        """Pictogram getter"""
        return self.context.icon_class

    user_icon = None


#
# Internal links
#

@factory_config(IInternalLink)
class InternalLink(InternalReferenceMixin, BaseLink):
    """Internal link persistent class"""

    icon_class = INTERNAL_LINK_ICON_CLASS
    icon_hint = INTERNAL_LINK_ICON_HINT

    _reference = FieldProperty(IInternalLink['reference'])
    force_canonical_url = FieldProperty(IInternalLink['force_canonical_url'])

    def is_visible(self, request=None):
        """Link visibility getter"""
        target = self.get_target()
        if target is not None:
            publication_info = IWorkflowPublicationInfo(target, None)
            if publication_info is not None:
                return publication_info.is_visible(request)
        return False

    def get_editor_url(self):
        """Editor URL getter"""
        return 'oid://{0}'.format(self.reference)

    def get_url(self, request=None, view_name=None):
        """URL getter"""
        target = self.get_target()
        if target is not None:
            if request is None:
                request = check_request()
            params = None
            if IInternalLinkCustomInfoTarget.providedBy(target):
                custom_info = IInternalLinkCustomInfo(self, None)
                if custom_info is not None:
                    params = custom_info.get_url_params()
                    if params:
                        params = urlencode(params)
            if self.force_canonical_url:
                return canonical_url(target, request, view_name, query=params)
            return relative_url(target, request, view_name=view_name, query=params)
        return ''


@subscriber(IObjectAddedEvent, context_selector=IInternalLink)
def handle_new_internal_link(event):
    """Check if link target is providing custom info"""
    link = event.object
    target = link.target
    if target is not None:
        info = IInternalLinkCustomInfoTarget(target, None)
        if info is not None:
            alsoProvides(link, info.internal_link_marker_interface)


@subscriber(IObjectModifiedEvent, context_selector=IInternalLink)
def handle_updated_internal_link(event):
    """Check when modified if new link target is providing custom info"""
    link = event.object
    # remove previous provided interfaces
    ifaces = tuple([
        iface for iface in directlyProvidedBy(link)
        if issubclass(iface, IInternalLinkCustomInfo)
    ])
    for iface in ifaces:
        noLongerProvides(link, iface)
    target = link.target
    if target is not None:
        info = IInternalLinkCustomInfoTarget(target, None)
        if info is not None:
            alsoProvides(link, info.internal_link_marker_interface)


@adapter_config(required=IInternalReference,
                provides=IAssociationInfo)
class InternalLinkAssociationInfoAdapter(BaseLinkInfoAdapter):
    """Internal link association info adapter"""

    @property
    def user_title(self):
        """User title getter"""
        title = II18n(self.context).query_attribute('title')
        if not title:
            target = self.context.get_target()
            if target is not None:
                title = II18n(target).query_attribute('title')
        return title or MISSING_INFO

    @property
    def user_header(self):
        """User header getter"""
        description = II18n(self.context).query_attribute('description')
        if not description:
            target = self.context.get_target()
            if (target is not None) and hasattr(target, 'header'):
                description = II18n(target).query_attribute('header')
        return description

    @property
    def inner_title(self):
        """Inner title getter"""
        target = self.context.get_target()
        if target is not None:
            sequence = ISequentialIdInfo(target)
            return '{0} ({1})'.format(II18n(target).query_attribute('title'),
                                      sequence.get_short_oid())
        return MISSING_INFO

    @property
    def human_size(self):
        """Human size getter"""
        return MISSING_INFO


#
# External links
#

@factory_config(IExternalLink)
class ExternalLink(BaseLink):
    """External link persistent class"""

    icon_class = EXTERNAL_LINK_ICON_CLASS
    icon_hint = EXTERNAL_LINK_ICON_HINT

    url = FieldProperty(IExternalLink['url'])
    language = FieldProperty(IExternalLink['language'])

    def get_editor_url(self):
        """Editor URL getter"""
        return self.url

    def get_url(self, request=None, view_name=None):
        """URL getter"""
        return self.url


@adapter_config(required=IExternalLink,
                provides=IAssociationInfo)
class ExternalLinkAssociationInfoAdapter(BaseLinkInfoAdapter):
    """External link association info adapter"""

    @property
    def user_title(self):
        """User title getter"""
        title = II18n(self.context).query_attribute('title')
        if not title:
            title = self.context.url
        return title or MISSING_INFO

    @property
    def user_header(self):
        """User header getter"""
        return II18n(self.context).query_attribute('description')

    @property
    def inner_title(self):
        """Inner title getter"""
        return self.context.url

    @property
    def human_size(self):
        """Human size getter"""
        return MISSING_INFO


#
# Mailto links
#

@factory_config(IMailtoLink)
class MailtoLink(BaseLink):
    """Mailto link persistent class"""

    icon_class = MAILTO_LINK_ICON_CLASS
    icon_hint = MAILTO_LINK_ICON_HINT

    address = FieldProperty(IMailtoLink['address'])
    address_name = FieldProperty(IMailtoLink['address_name'])

    def get_editor_url(self):
        """Editor URL getter"""
        return 'mailto:{0} <{1}>'.format(self.address_name, self.address)

    def get_url(self, request=None, view_name=None):
        """URL getter"""
        return 'mailto:{}'.format(url_quote('{} <{}>'.format(self.address_name, self.address))
                                  if self.address_name else self.address)


@adapter_config(required=IMailtoLink,
                provides=IAssociationInfo)
class MailtoLinkAssociationInfoAdapter(BaseLinkInfoAdapter):
    """Mailto link association info adapter"""

    @property
    def user_title(self):
        """User title getter"""
        title = II18n(self.context).query_attribute('title')
        if not title:
            title = self.context.address_name
        return title or MISSING_INFO

    @property
    def user_header(self):
        """User header getter"""
        return II18n(self.context).query_attribute('description')

    @property
    def inner_title(self):
        """Inner title getter"""
        if self.context.address_name:
            return escape('{} <{}>'.format(self.context.address_name, self.context.address))
        return self.context.address

    @property
    def human_size(self):
        """Human size getter"""
        return MISSING_INFO



#
# Copyright (c) 2015-2024 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from zope.interface import Interface

from pyams_content.shared.view.portlet.skin import IViewItemsPortletPanelsRendererSettings
from pyams_form.field import Fields
from pyams_form.group import Group
from pyams_form.interfaces.form import IFormFields, IGroup
from pyams_portal.zmi.interfaces import IPortletRendererSettingsEditForm
from pyams_utils.adapter import adapter_config
from pyams_zmi.form import FormGroupChecker
from pyams_zmi.interfaces import IAdminLayer

__docformat__ = 'restructuredtext'

from pyams_content import _


@adapter_config(required=(IViewItemsPortletPanelsRendererSettings, IAdminLayer, IPortletRendererSettingsEditForm),
                provides=IFormFields)
def view_items_panels_renderer_settings_fields(context, request, form):
    return Fields(Interface)


@adapter_config(name='illustration',
                required=(IViewItemsPortletPanelsRendererSettings, IAdminLayer, IPortletRendererSettingsEditForm),
                provides=IGroup)
class ViewItemsPanelsRendererIllustrationSettingsGroup(FormGroupChecker):
    """View item portlet panels renderer illustration settings group"""

    fields = Fields(IViewItemsPortletPanelsRendererSettings).select('display_illustrations', 'thumb_selection')
    weight = 10


@adapter_config(name='header',
                required=(IViewItemsPortletPanelsRendererSettings, IAdminLayer, IPortletRendererSettingsEditForm),
                provides=IGroup)
class ViewItemsPanelsRendererHeaderSettingsGroup(Group):
    """View items portlet panels renderer header settings group"""

    legend = _("Header display")

    fields = Fields(IViewItemsPortletPanelsRendererSettings).select('header_display_mode', 'start_length')
    weight = 20


@adapter_config(name='pagination',
                required=(IViewItemsPortletPanelsRendererSettings, IAdminLayer, IPortletRendererSettingsEditForm),
                provides=IGroup)
class ViewItemsPanelsRendererPaginationSettingsGroup(FormGroupChecker):
    """View item portlet panels renderer pagination settings group"""

    fields = Fields(IViewItemsPortletPanelsRendererSettings).select('paginate', 'page_size')
    weight = 30



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.history module

This module provides persistent classes for history management.

History can be used to store state of any object implementing IHistoryTarget
interface; it is generally used before object updates, to store current state.
A comment can be associated to each history item, as well as a copy of any
email message which could has been sent during the operation.
"""

from datetime import datetime

from persistent import Persistent
from zope.container.btree import BTreeContainer
from zope.container.contained import Contained
from zope.copy import copy
from zope.schema.fieldproperty import FieldProperty

from pyams_content.feature.history.interfaces import HISTORY_CONTAINER_KEY, IHistoryContainer, IHistoryItem, \
    IHistoryTarget
from pyams_utils.adapter import adapter_config, get_annotation_adapter
from pyams_utils.factory import create_object, factory_config
from pyams_utils.timezone import gmtime, tztime

__docformat__ = 'restructuredtext'


@factory_config(IHistoryItem)
class HistoryItem(Persistent, Contained):
    """History item"""

    _timestamp = FieldProperty(IHistoryItem['timestamp'])
    principal = FieldProperty(IHistoryItem['principal'])
    message = FieldProperty(IHistoryItem['message'])
    comment = FieldProperty(IHistoryItem['comment'])
    _value = FieldProperty(IHistoryItem['value'])

    @property
    def timestamp(self):
        return self._timestamp

    @timestamp.setter
    def timestamp(self, value):
        self._timestamp = gmtime(value)

    @property
    def value(self):
        return self._value

    @value.setter
    def value(self, value):
        self._value = copy(value)


@factory_config(IHistoryContainer)
class HistoryContainer(BTreeContainer):
    """History container"""

    def add_history(self, context, comment=None, message=None, request=None):
        """Add context history"""
        item = create_object(IHistoryItem)
        if item is not None:
            item.timestamp = tztime(datetime.utcnow())
            if request is not None:
                item.principal = request.principal.id
            item.message = message
            item.comment = comment
            item.value = context
            self[str(item.timestamp.timestamp())] = item


@adapter_config(required=IHistoryTarget,
                provides=IHistoryContainer)
def history_target_container(context):
    """History target container"""
    return get_annotation_adapter(context, HISTORY_CONTAINER_KEY, IHistoryContainer)



#
# Copyright (c) 2015-2022 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.zmi.html module

This module provides configuration adapter for TinyMCE HTML editor.
"""

__docformat__ = 'restructuredtext'

import json

from zope.dublincore.interfaces import IZopeDublinCore

from pyams_content.zmi import content_js
from pyams_layer.interfaces import ISkinnable
from pyams_skin.interfaces.widget import IHTMLWidget
from pyams_utils.adapter import ContextAdapter, adapter_config
from pyams_utils.fanstatic import get_resource_path
from pyams_utils.interfaces.data import IObjectDataRenderer
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url


@adapter_config(required=IHTMLWidget,
                provides=IObjectDataRenderer)
class HTMLWidgetDataRenderer(ContextAdapter):
    """HTML widget data getter"""

    def get_object_data(self):
        """Object data getter"""
        context = self.context.context
        request = self.context.request
        data = {
            'ams-modules': {
                'content': get_resource_path(content_js)
            },
            'ams-tinymce-init-callback': 'MyAMS.content.TinyMCE.initEditor',
            'ams-tinymce-link-list': absolute_url(context, request, 'get-links-list.json')
        }
        skinnable = get_parent(request.context, ISkinnable)
        if skinnable is not None:
            editor_stylesheet = skinnable.editor_stylesheet
            if editor_stylesheet:
                modified = IZopeDublinCore(editor_stylesheet).modified
                data['ams-tinymce-content-css'] = absolute_url(editor_stylesheet,
                                                               request,
                                                               query={
                                                                   '_': modified.timestamp()
                                                               })
        return json.dumps(data)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_content.feature.navigation.portlet.skin module

"""

from persistent import Persistent
from zope.container.contained import Contained
from zope.interface import Interface
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.association.interfaces import IAssociationInfo
from pyams_content.feature.navigation.portlet import IDoubleNavigationPortletSettings, \
    ISimpleNavigationPortletSettings
from pyams_content.feature.navigation.portlet.skin.interfaces import \
    IDoubleNavigationPortletDefaultRendererSettings, \
    ISimpleNavigationPortletDefaultRendererSettings
from pyams_layer.interfaces import IPyAMSLayer
from pyams_portal.interfaces import IPortalContext, IPortletRenderer
from pyams_portal.skin import PortletRenderer
from pyams_template.template import template_config
from pyams_utils.adapter import adapter_config
from pyams_utils.factory import factory_config


__docformat__ = 'restructuredtext'

from pyams_content import _


class BaseSimpleNavigationPortletRenderer(PortletRenderer):
    """Base simple navigation portlet renderer"""

    @staticmethod
    def get_link_info(link):
        """Link information getter"""
        return IAssociationInfo(link)


@factory_config(ISimpleNavigationPortletDefaultRendererSettings)
class SimpleNavigationPortletDefaultRendererSettings(Persistent, Contained):
    """Simple navigation portlet default renderer settings"""

    thumb_selection = FieldProperty(
        ISimpleNavigationPortletDefaultRendererSettings['thumb_selection'])


@adapter_config(required=(IPortalContext, IPyAMSLayer, Interface,
                          ISimpleNavigationPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/simple-default.pt', layer=IPyAMSLayer)
class SimpleNavigationPortletDefaultRenderer(BaseSimpleNavigationPortletRenderer):
    """Simple navigation portlet default renderer"""

    label = _("Horizontal grid links with illustrations (default)")

    settings_interface = ISimpleNavigationPortletDefaultRendererSettings
    weight = 10


@factory_config(IDoubleNavigationPortletDefaultRendererSettings)
class DoubleNavigationPortletDefaultRendererSettings(Persistent, Contained):
    """Double navigation portlet default renderer settings"""

    thumb_selection = FieldProperty(
        IDoubleNavigationPortletDefaultRendererSettings['thumb_selection'])


@adapter_config(required=(IPortalContext, IPyAMSLayer, Interface,
                          IDoubleNavigationPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/double-default.pt', layer=IPyAMSLayer)
class DoubleNavigationPortletDefaultRenderer(PortletRenderer):
    """Double navigation portlet default renderer"""

    label = _("Horizontal tabs of grids links with illustrations (default)")

    settings_interface = IDoubleNavigationPortletDefaultRendererSettings
    weight = 10

    @staticmethod
    def get_link_info(link):
        """Link information getter"""
        return IAssociationInfo(link)
